#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on 2023/07/01
@author: juliengautier
window for crop image
"""

from PyQt6.QtWidgets import QApplication, QVBoxLayout, QWidget
from PyQt6.QtWidgets import QLabel, QMainWindow, QFileDialog, QSizePolicy, QStatusBar
from PyQt6 import QtCore, QtGui
from PyQt6.QtCore import Qt
from PyQt6.QtGui import QAction
from PyQt6.QtGui import QIcon, QShortcut
from PIL import Image
import sys
import time
import pyqtgraph as pg  # pyqtgraph biblio permettent l'affichage
import numpy as np
import qdarkstyle  # pip install qdakstyle https://github.com/ColinDuquesnoy/QDarkStyleSheet  sur conda
import os
import pathlib


class WINCROP(QMainWindow):
    
    def __init__(self, parent=None, conf=None, name='VISU'):
        
        super().__init__()
        self.name = name
        self.parent = parent
        p = pathlib.Path(__file__)
        if conf is None:
            self.conf = QtCore.QSettings(str(p.parent / 'confVisu.ini'), QtCore.QSettings.Format.IniFormat)
        else:
            self.conf = conf
        sepa = os.sep
        self.icon = str(p.parent) + sepa+'icons' + sepa
        self.path = self.conf.value(self.name+"/path")
        self.isWinOpen = False
        self.setWindowTitle('CropWindows')
        self.setWindowIcon(QIcon(self.icon+'LOA.png'))
        self.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
        self.left = 100
        self.top = 30
        self.width = 800
        self.height = 800
        self.setGeometry(self.left, self.top, self.width, self.height)
        self.dimx = 1200
        self.dimy = 900
        
        self.setWindowIcon(QIcon('./icons/LOA.png'))
        
        self.data = (40*np.random.rand(self.dimx, self.dimy)).round()
        self.setup()
        self.shortcut()
        
    def setup(self):
        
        self.toolBar = self.addToolBar('tools')
        self.toolBar.setMovable(False)

        menubar = self.menuBar()
        menubar.setNativeMenuBar(False)
        self.statusBar = QStatusBar()
        self.setContentsMargins(0, 0, 0, 0)
        
        self.setStatusBar(self.statusBar)
        self.fileMenu = menubar.addMenu('&File')
        self.ImageMenu = menubar.addMenu('&Image')
        self.optionMenu = menubar.addMenu('&Options')
        self.openAct = QAction(QtGui.QIcon(self.icon+"Open.png"), 'Open File', self)
        self.openAct.setShortcut('Ctrl+o')
        self.toolBar.addAction(self.openAct)
        self.openAct.triggered.connect(self.OpenF)
        
        self.fileMenu.addAction(self.openAct)
        
        self.saveAct = QAction(QtGui.QIcon(self.icon+"disketteSave.png"), 'Save file', self)
        self.saveAct.setShortcut('Ctrl+s')
        self.toolBar.addAction(self.saveAct)
        self.saveAct.triggered.connect(self.SaveF)
        
        self.fileMenu.addAction(self.saveAct)
    
        self.checkBoxScale = QAction(QtGui.QIcon(self.icon+"expand.png"), ' Auto Scale on', self)
        self.checkBoxScale.setCheckable(True)
        self.checkBoxScale.setChecked(True)
        self.toolBar.addAction(self.checkBoxScale)
        self.ImageMenu.addAction(self.checkBoxScale)
        self.checkBoxScale.triggered.connect(self.checkBoxScaleImage)

        self.label_CrossValue = QLabel()
        self.label_CrossValue.setStyleSheet("font:13pt")
        
        self.label_Cross = QLabel()
        # self.label_Cross.setMaximumHeight(20)
        self.label_Cross.setMaximumWidth(170)
        self.label_Cross.setStyleSheet("font:12pt")
        self.statusBar.addPermanentWidget(self.label_Cross)
        self.statusBar.addPermanentWidget(self.label_CrossValue)

        self.vbox2 = QVBoxLayout()
        self.winImage = pg.GraphicsLayoutWidget()
        self.winImage.setContentsMargins(0, 0, 0, 0)
        self.winImage.setAspectLocked(True)
        self.winImage.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
        self.winImage.ci.setContentsMargins(0, 0, 0, 0)
        self.vbox2.addWidget(self.winImage)
        self.vbox2.setContentsMargins(0, 0, 0, 0)
        
        self.p1 = self.winImage.addPlot()
        self.imh = pg.ImageItem()
        self.axeX = self.p1.getAxis('bottom')
        self.axeY = self.p1.getAxis('left')
        self.p1.addItem(self.imh)
        self.p1.setMouseEnabled(x=False, y=False)
        self.p1.setContentsMargins(0, 0, 0, 0)
    
        self.p1.setAspectLocked(True, ratio=1)
        self.p1.showAxis('right', show=False)
        self.p1.showAxis('top', show=False)
        self.p1.showAxis('left', show=False)
        self.p1.showAxis('bottom', show=False)

        MainWidget = QWidget()
        MainWidget.setLayout(self.vbox2)
        self.setCentralWidget(MainWidget)

        # histogram
        self.hist = pg.HistogramLUTItem()
        self.hist.setImageItem(self.imh)
        self.hist.autoHistogramRange()
        self.hist.gradient.loadPreset('flame')

        if self.parent is not None:
            # if signal emit in another thread (see visual)
            self.parent.signalCrop.connect(self.Display)

    def checkBoxScaleImage(self):

        if self.checkBoxScale.isChecked():
            self.checkBoxScale.setIcon(QtGui.QIcon(self.icon+"expand.png"))
            self.checkBoxScale.setText('Auto Scale On')
        else:
            self.checkBoxScale.setIcon(QtGui.QIcon(self.icon+"minimize.png"))
            self.checkBoxScale.setText('Auto Scale Off')

    def Display(self, data, stepX=1, stepY=1):
        
        self.data = data
        self.dimx = self.data.shape[0]
        self.dimy = self.data.shape[1]
        self.stepX = stepX
        self.stepY = stepY
        if self.checkBoxScale.isChecked():
            self.imh.setImage(self.data, autoLevels=True, autoDownsample=True)
        else:
            self.imh.setImage(self.data, autoLevels=False, autoDownsample=True)

    def shortcut(self):
        # keyboard shortcut
        
        self.shortcutPu = QShortcut(QtGui.QKeySequence("+"), self)
        self.shortcutPu.activated.connect(self.paletteup)
        self.shortcutPu.setContext(Qt.ShortcutContext(3))
        # 3: The shortcut is active when its parent widget, or any of its children has focus. default O The shortcut is active when its parent widget has focus.
        self.shortcutPd = QtGui.QShortcut(QtGui.QKeySequence("-"), self)
        self.shortcutPd.activated.connect(self.palettedown)
        self.shortcutPd.setContext(Qt.ShortcutContext(3))
        # mousse action:
        self.proxy = pg.SignalProxy(self.p1.scene().sigMouseMoved, rateLimit=60, slot=self.mouseMoved)
        # self.p1.scene().sigMouseClicked.connect(self.mouseClick)
        # self.vb=self.p1.vb

    def paletteup(self):
        # change the color scale
        levels = self.imh.getLevels()
        if levels[0] is None:
            xmax = self.data.max()
            xmin = self.data.min()
        else:
            xmax = levels[1]
            xmin = levels[0]
        self.imh.setLevels([xmin, xmax-(xmax - xmin) / 10])
        self.hist.setHistogramRange(xmin, xmax)

    def palettedown(self):
        levels = self.imh.getLevels()
        if levels[0] is None:
            xmax = self.data.max()
            xmin = self.data.min()
        else:
            xmax = levels[1]
            xmin = levels[0]
        self.imh.setLevels([xmin, xmax + (xmax - xmin) / 10])
        # hist.setImageItem(imh,clear=True)
        self.hist.setHistogramRange(xmin, xmax)

    def OpenF(self, fileOpen=False):

        fname = QFileDialog.getOpenFileNames(self, "Open File")
        self.openedFiles = fname[0]
        fichier = self.openedFiles[0]
        ext = os.path.splitext(fichier)[1]
        
        if ext == '.txt':  # text file
            data = np.loadtxt(str(fichier))
        elif ext == '.TIFF' or ext == '.tif' or ext == '.Tiff' or ext == '.jpg' or ext == '.JPEG' or ext == '.png':  # tiff File
            dat = Image.open(fichier)
            data = np.array(dat)
            data = np.rot90(data, 3)

        self.Display(self.data)

    def SaveF(self):
        # save as tiff
        fname = QFileDialog.getSaveFileName(self, "Save data as TIFF", self.path)
        self.path = os.path.dirname(str(fname[0]))
        fichier = fname[0]
        print(fichier, ' is saved')
        self.conf.setValue(self.name+"/path", self.path)
        time.sleep(0.1)
        self.dataS = np.rot90(self.data, 1)
        img_PIL = Image.fromarray(self.dataS)
        img_PIL.save(str(fname[0])+'.TIFF', format='TIFF')
        
    def mouseMoved(self, evt):
        pos = evt[0]  # using signal proxy turns original arguments into a tuple
        if self.p1.sceneBoundingRect().contains(pos):
            mousePoint = self.p1.vb.mapSceneToView(pos)
            self.xMouse = (mousePoint.x())
            self.yMouse = (mousePoint.y())
            if ((self.xMouse > 0 and self.xMouse < self.dimx-1) and (self.yMouse > 0 and self.yMouse < self.dimy - 1)):
                self.xc = self.xMouse
                self.yc = self.yMouse
                try:
                    dataCross = self.data[int(self.xc), int(self.yc)]
                except:
                    dataCross = 0  # evoid to have an error if cross if out of the image
                    self.xc = 0
                    self.yc = 0
                self.label_Cross.setText('x=' + str(int(self.xc)) + ' y=' + str(int(self.yc)))
                dataCross = round(dataCross, 3)  # take data  value  on the cross
                self.label_CrossValue.setText(' v.=' + str(dataCross))
        
    def closeEvent(self, event):
        """ when closing the window
        """
        self.isWinOpen = False
        time.sleep(0.1)
        event.accept()
    
        
if __name__ == "__main__":
    appli = QApplication(sys.argv)
    appli.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
    e = WINCROP(name='VISU')
    e.show()
    appli.exec_()






# Zhenpeng Zhou <zhenp3ngzhou cir{a} gmail dot com>
# Fri Nov 17 2017


import os
import time
import numpy as np


class SIFFile(object):
    """
    A class that reads the contents and metadata of an Andor .sif file. Compatible with images as well as spectra.
    Exports data as numpy array or xarray.DataArray.
    Example: SIFFile('my_spectrum.sif').read_all()
    In addition to the raw data, SIFFile objects provide a number of meta data variables:
    :ivar x_axis: the horizontal axis (can be pixel numbers or wavelength in nm)
    :ivar original_filename: the original file name of the .sif file
    :ivar date: the date the file was recorded
    :ivar model: camera model
    :ivar temperature: sensor temperature in degrees Celsius
    :ivar exposuretime: exposure time in seconds
    :ivar cycletime: cycle time in seconds
    :ivar accumulations: number of accumulations
    :ivar readout: pixel readout rate in MHz
    :ivar xres: horizontal resolution
    :ivar yres: vertical resolution
    :ivar width: image width
    :ivar height: image height
    :ivar xbin: horizontal binning
    :ivar ybin: vertical binning
    :ivar gain: EM gain level
    :ivar vertical_shift_speed: vertical shift speed
    :ivar pre_amp_gain: pre-amplifier gain
    :ivar stacksize: number of frames
    :ivar filesize: size of the file in bytes
    :ivar m_offset: offset in the .sif file to the actual data
    """

    def __init__(self, filepath):
        self.filepath = filepath
        self._read_header(filepath)

    def __repr__(self):
        info = (('Original Filename', self.original_filename),
                ('Date', self.date),
                ('Camera Model', self.model),
                ('Temperature (deg.C)', '{:f}'.format(self.temperature)),
                ('Exposure Time', '{:f}'.format(self.exposuretime)),
                ('Cycle Time', '{:f}'.format(self.cycletime)),
                ('Number of accumulations', '{:d}'.format(self.accumulations)),
                ('Pixel Readout Rate (MHz)', '{:f}'.format(self.readout)),
                ("Horizontal Camera Resolution", '{:d}'.format(self.xres)),
                ("Vertical Camera Resolution", '{:d}'.format(self.yres)),
                ("Image width", '{:d}'.format(self.width)),
                ("Image Height", '{:d}'.format(self.height)),
                ("Horizontal Binning", '{:d}'.format(self.xbin)),
                ("Vertical Binning", '{:d}'.format(self.ybin)),
                ("EM Gain level", '{:f}'.format(self.gain)),
                ("Vertical Shift Speed", '{:f}'.format(self.vertical_shift_speed)),
                ("Pre-Amplifier Gain", '{:f}'.format(self.pre_amp_gain)),
                ("Stacksize", '{:d}'.format(self.stacksize)),
                ("Filesize", '{:d}'.format(self.filesize)),
                ("Offset to Image Data", '{:f}'.format(self.m_offset)))
        desc_len = max([len(d) for d in list(zip(*info))[0]]) + 3
        res = ''
        for description, value in info:
            res += ('{:' + str(desc_len) + '}{}\n').format(description + ': ', value)

        res = super().__repr__() + '\n' + res
        return res

    def _read_header(self, filepath):
        f = open(filepath, 'rb')
        headerlen = 32
        spool = 0
        i = 0
        while i < headerlen + spool:
            line = f.readline().strip()
            if i == 0:
                if line != b'Andor Technology Multi-Channel File':
                    f.close()
                    raise Exception('{} is not an Andor SIF file'.format(filepath))
            elif i == 2:
                tokens = line.split()
                self.temperature = float(tokens[5])
                self.date = time.strftime('%c', time.localtime(float(tokens[4])))
                self.exposuretime = float(tokens[12])
                self.cycletime = float(tokens[13])
                self.accumulations = int(tokens[15])
                self.readout = 1 / float(tokens[18]) / 1e6
                self.gain = float(tokens[21])
                self.vertical_shift_speed = float(tokens[41])
                self.pre_amp_gain = float(tokens[43])
            elif i == 3:
                self.model = line.decode('utf-8')
            elif i == 5:
                self.original_filename = line.decode('utf-8')
            elif i == 7:
                tokens = line.split()
                if len(tokens) >= 1 and tokens[0] == 'Spooled':
                    spool = 1
            if i == 9:
                wavelength_info = line.split()
                self.center_wavelength = float(wavelength_info[3])
                self.grating = float(wavelength_info[6])
                self.grating_blaze = float(wavelength_info[7])
            if i == 19:
                self.wavelength_coefficients = [float(num) for num in line.split()][::-1]
            if 7 < i < headerlen - 12:
                if len(line) == 17 and line[0:6] == b'65539 ':
                    # and line[7] == b'x01' and line[8] == b'x20' \
                    # and line[9] == b'x00':
                    headerlen = i + 12
            if i == headerlen - 2:
                if line[:12] == b'Pixel number':
                    line = line[12:]
                tokens = line.split()
                if len(tokens) < 6:
                    raise Exception('Not able to read stacksize.')
                self.yres = int(tokens[2])
                self.xres = int(tokens[3])
                self.stacksize = int(tokens[5])
            elif i == headerlen - 1:
                tokens = line.split()
                if len(tokens) < 7:
                    raise Exception("Not able to read Image dimensions.")
                self.left = int(tokens[1])
                self.top = int(tokens[2])
                self.right = int(tokens[3])
                self.bottom = int(tokens[4])
                self.xbin = int(tokens[5])
                self.ybin = int(tokens[6])
            i += 1

        f.close()

        width = self.right - self.left + 1
        mod = width % self.xbin
        self.width = int((width - mod) / self.ybin)
        height = self.top - self.bottom + 1
        mod = height % self.ybin
        self.height = int((height - mod) / self.xbin)

        self.filesize = os.path.getsize(filepath)
        self.datasize = self.width * self.height * 4 * self.stacksize
        self.m_offset = self.filesize - self.datasize - 8

        self.x_axis = np.polyval(self.wavelength_coefficients, np.arange(self.left, self.right + 1))

    def read_block(self, num=0):
        """
        Returns a specific block (i.e. frame) in the .sif file as a numpy array.
        :param num: block number
        :return: a numpy array with shape (y, x)
        """
        f = open(self.filepath, 'rb')
        f.seek(self.m_offset + num * self.width * self.height * 4)
        block = f.read(self.width * self.height * 4)
        data = np.fromstring(block, dtype=np.float32)
        f.close()
        return data.reshape(self.height, self.width)

    def read_all(self):
        """
        Returns all blocks (i.e. frames) in the .sif file as a numpy array.
        :return: a numpy array with shape (blocks, y, x)
        """
        f = open(self.filepath, 'rb')
        f.seek(self.m_offset)
        block = f.read(self.width * self.height * self.stacksize * 4)
        data = np.fromstring(block, dtype=np.float32)
        f.close()
        return data.reshape(self.stacksize, self.height, self.width)

    def as_xarray(self, x_axis_quantity='wavelength'):
        """
        Returns an xarray.DataArray object containing all frames, all metadata and all coordinate axis values. If the
        file contains a spectrum, the wavelength axis can be converted to wavenumbers or photon energy.  This method
        requires the xarray package to be installed.
        :param x_axis_quantity: Only relevant for spectra. Can be either 'wavelength' (default), 'wavenumber' or \
        'photon energy'.
        :return: An xarray.DataArray containing the entire contents of the .sif file.
        """
        try:
            import xarray as xr
            data = self.read_all()
            y_axis = np.arange(data.shape[1])
            # determine if it's an image or a spectrum: check if the x_axis spacing is always one
            if (np.abs((np.diff(self.x_axis) - 1)) < 1e-5).all():
                # it's an image
                x_axis = self.x_axis.astype(np.uint16)
                x_axis_quantity = 'x'
                x_unit = 'px'
                x_name = 'x'
            else:
                # it's a spectrum
                if x_axis_quantity == 'wavelength':
                    x_axis = self.x_axis
                    x_unit = 'nm'
                    x_name = 'Wavelength'
                elif x_axis_quantity == 'wavenumber':
                    x_axis = (1e7 / self.x_axis)[::-1]
                    data = np.flip(data, 2)
                    x_unit = 'cm^-1'
                    x_name = 'Wavenumber'
                elif x_axis_quantity == 'photon_energy':
                    x_axis = (1239.84 / self.x_axis)[::-1]
                    data = np.flip(data, 2)
                    x_unit = 'eV'
                    x_name = 'Photon energy'
                else:
                    raise RuntimeError('X-axis quantity "{}" not recognized!'.format(x_axis_quantity))

            if data.shape[0] == 1:
                # Only one frame
                data = np.transpose(data[0])
                data_array = xr.DataArray(data, coords=[(x_axis_quantity, x_axis), ('y', y_axis)],
                                    name='intensity')
            else:
                # multiple frames
                frame_axis = np.arange(data.shape[0])
                data = np.transpose(data, [2, 1, 0])
                data_array = xr.DataArray(data, coords=[(x_axis_quantity, x_axis), ('y', y_axis),
                                                  ('frames', frame_axis)], name='intensity')
                data_array.frames.attrs['long_name'] = 'Frame number'

            data_array.attrs['long_name'] = 'Intensity'
            data_array.attrs['units'] = 'arb. u.'
            data_array.y.attrs['units'] = 'px'
            data_array[x_axis_quantity].attrs['long_name'] = x_name
            data_array[x_axis_quantity].attrs['units'] = x_unit
            data_array.attrs['sif_metadata'] = str(self)

            return data_array

        except ImportError:
            raise RuntimeError("xarray package required for this method!")


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Dec 19 11:43:05 2018
@author: juliengautier

"""

from PyQt6.QtWidgets import (QApplication, QVBoxLayout, QHBoxLayout, QWidget, 
                             QLabel, QMainWindow, QMessageBox, 
                             QFileDialog, QInputDialog, QFrame, QGroupBox,
                             QPushButton, QLineEdit)

from PyQt6 import QtCore, QtGui 

from PyQt6.QtGui import QAction
from PyQt6.QtGui import QIcon
from PyQt6.QtCore import QThread, pyqtSignal, QTimer, Qt
import sys
import time
import pyqtgraph as pg
import numpy as np
import qdarkstyle
import pylab
import os
from scipy.ndimage import gaussian_filter
import pathlib
from scipy import ndimage
from collections import deque


class PointingWorker(QThread):
    results_ready = pyqtSignal(dict)
    
    def __init__(self):
        super().__init__()
        self.data = None
        self.stepX = 1
        self.stepY = 1
        self.xini = 0
        self.yini = 0
        self.use_center_of_mass = False
        self.should_stop = False
        self._has_new_data = False
        self._lock = QtCore.QMutex()

    def set_data(self, obje):
        self._lock.lock()
        self.data = obje[0]
        self.stepX = obje[1]
        self.stepY = obje[2]
        self.xini = obje[3]
        self.yini = obje[4]
        self._has_new_data = True
        self._lock.unlock()
    
    def run(self):
        """Boucle permanente"""
        while not self.should_stop:
            if self._has_new_data:
                self._lock.lock()
                data = self.data.copy()
                stepX = self.stepX
                stepY = self.stepY
                xini = self.xini    # prmet de se placer dans la coordonnées de l image entiere
                yini = self.yini
                self._has_new_data = False
                self._lock.unlock()
                
                try:
                    ds_threshold = 512
                    if data.shape[0] > ds_threshold or data.shape[1] > ds_threshold:
                        ds = 4
                        data_small = data[::ds, ::ds]   # Sous-échantillonnage pour accélérer le calcul du gausian 
                        dataF = gaussian_filter(data_small, 2)
                    else:
                        ds = 1
                        dataF = gaussian_filter(data, 5)

                    if self.use_center_of_mass:
                        (xec, yec) = ndimage.center_of_mass(dataF)
                        label = 'center of mass'
                    else:
                        (xec, yec) = pylab.unravel_index(dataF.argmax(), data_small.shape if ds > 1 else data.shape)
                        label = 'max'
                    
                    # Raffinement
                    margin = ds * 2
                    xc = int(round(xec * ds))
                    yc = int(round(yec * ds))
                    x0 = max(0, xc - margin)
                    x1 = min(data.shape[0], xc + margin)
                    y0 = max(0, yc - margin)
                    y1 = min(data.shape[1], yc + margin)
                    
                    if x1 > x0 and y1 > y0:
                        patch = data[x0:x1, y0:y1]
                        if self.use_center_of_mass:
                            (dx, dy) = ndimage.center_of_mass(patch)
                        else:
                            (dx, dy) = pylab.unravel_index(patch.argmax(), patch.shape)
                        xec = (x0 + dx + xini) * stepX
                        yec = (y0 + dy + yini) * stepY
                    else:
                        xec = (xec * ds + xini) * stepX
                        yec = (yec * ds + yini) * stepY
                    
                    if not self.should_stop:
                        self.results_ready.emit({
                            'xec': xec, 'yec': yec, 'label': label,
                            'stepX': stepX, 'stepY': stepY
                        })
                except Exception as e:
                    print(f"Erreur PointingWorker: {e}")
            else:
                self.msleep(10)

    def stop(self):
        self.should_stop = True
        self.wait(500)


class StatsCard(QFrame):
    """Widget carte pour afficher une statistique"""
    def __init__(self, title, color='white', parent=None):
        super().__init__(parent)
        self.setFrameShape(QFrame.Shape.StyledPanel)
        self.setStyleSheet(f"""
            QFrame {{
                background-color: #2b2b2b;
                border: 2px solid {color};
                border-radius: 8px;
                padding: 5px;
            }}
        """)
        
        layout = QVBoxLayout()
        layout.setSpacing(3)
        layout.setContentsMargins(5, 5, 5, 5)
        
        # Titre
        self.titleLabel = QLabel(title)
        self.titleLabel.setStyleSheet(f"color: {color}; font: bold 9pt; border: none;")
        self.titleLabel.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.titleLabel.setWordWrap(False)  # Pas de retour à la ligne
        
        # Valeur
        self.valueLabel = QLabel('--')
        self.valueLabel.setStyleSheet(f"color: {color}; font: bold 11pt; border: none;")  # Réduit de 14pt à 11pt
        self.valueLabel.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.valueLabel.setWordWrap(False)  # Pas de retour à la ligne
        
        layout.addWidget(self.titleLabel)
        layout.addWidget(self.valueLabel)
        self.setLayout(layout)
    
    def setValue(self, value):
        """Mettre à jour la valeur affichée"""
        self.valueLabel.setText(str(value))


class WINPOINTING(QMainWindow):
    
    def __init__(self, parent=None, conf=None, name='VISU'):
        
        super().__init__()

        self.name = name
        self.parent = parent
        p = pathlib.Path(__file__)
        if conf is None:
            self.conf = QtCore.QSettings(str(p.parent / 'confVisu.ini'), QtCore.QSettings.Format.IniFormat)
        else:
            self.conf = conf
        
        sepa = os.sep
        self.icon = str(p.parent) + sepa + 'icons' + sepa
        self.path = self.conf.value(self.name+"/path")
        self.isWinOpen = False
        self.setWindowTitle('Pointing Stability Monitor')
        self.setWindowIcon(QIcon(self.icon+'LOA.png'))
        self.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
        self.left = 100
        self.top = 30
        self.width = 1400
        self.height = 800
        self.setGeometry(self.left, self.top, self.width, self.height)
        self.dimx = 1200
        self.dimy = 900
        self.bloqq = 1
        self.xec = int(self.conf.value(self.name+"/xec")) if self.conf.value(self.name+"/xec") else 0
        self.yec = int(self.conf.value(self.name+"/yec")) if self.conf.value(self.name+"/yec") else 0
        self.r1x = int(self.conf.value(self.name+"/r1x")) if self.conf.value(self.name+"/r1x") else 0
        self.r1y = int(self.conf.value(self.name+"/r1y")) if self.conf.value(self.name+"/r1y") else 0
        self.r2 = int(self.conf.value(self.name+"/r2x")) if self.conf.value(self.name+"/r2x") else 0
        self.r2 = int(self.conf.value(self.name+"/r2y")) if self.conf.value(self.name+"/r2y") else 0
        
        self.kE = 0
        #  self.time_old = time.time()
        
        self._max_points = 1000  # Valeur par défaut
        
        # Utiliser deque au lieu de listes - BEAUCOUP PLUS EFFICACE !
        self.Xec = deque(maxlen=self._max_points)
        self.Yec = deque(maxlen=self._max_points)
        self.timestamps = deque(maxlen=self._max_points)

        self.fwhmX = 100
        self.fwhmY = 100
        self.E = []
        
        # Variables pour la mise à jour des graphiques
        self.current_label = ''
        self.current_stepX = 1
        self.current_stepY = 1
        self.data_updated = False  # Flag pour savoir si de nouvelles données sont disponibles
        
        # Créer le worker thread
        self.worker = PointingWorker()
        self.worker.results_ready.connect(self.update_display)
        self.worker.start() 
        # Timer pour la mise à jour des graphiques
        self.plot_update_timer = QTimer()
        self.plot_update_timer.timeout.connect(self.updatePlotsFromTimer)
        self.plot_update_interval = 200  # ms (5 fps par défaut)
        
        
        # Compteur pour ne mettre à jour QUE via le timer
        self.update_counter = 0
        self.update_every_n_points = 5  # Tous les 5 points
        
        # Flag pour indiquer si de nouvelles données sont arrivées
        self.new_data_available = False
        
        # Create x and y indices
        x = np.arange(0, self.dimx)
        y = np.arange(0, self.dimy)
        y, x = np.meshgrid(y, x)
    
        self.data = (40*np.random.rand(self.dimx, self.dimy)).round()
        self.winFFT = WinPointingFFT(parent=self)
        self.setup()
    
    @property
    def max_points(self):
        """Getter pour max_points"""
        return self._max_points
    
    @max_points.setter
    def max_points(self, value):
        """Setter pour max_points - recrée les deques avec la nouvelle taille"""
        self._max_points = value
        # Recréer les deques avec la nouvelle taille maximale
        old_xec = list(self.Xec)
        old_yec = list(self.Yec)
        self.Xec = deque(old_xec[-value:], maxlen=value)
        self.Yec = deque(old_yec[-value:], maxlen=value)
        
    def setup(self):
        
        menubar = self.menuBar()
        menubar.setNativeMenuBar(False)
        self.fileMenu = menubar.addMenu('&File')
        self.optionMenu = menubar.addMenu('&Options')
        
        self.openAct = QAction(QtGui.QIcon(self.icon+"Open.png"), 'Open File', self)
        self.openAct.setShortcut('Ctrl+o')
        self.openAct.triggered.connect(self.OpenF)
        self.fileMenu.addAction(self.openAct)
        
        self.saveAct = QAction(QtGui.QIcon(self.icon+"disketteSave.png"), 'Save file', self)
        self.saveAct.setShortcut('Ctrl+s')
        self.saveAct.triggered.connect(self.SaveF)
        self.fileMenu.addAction(self.saveAct)
        
        # Options menu
        self.resetButton = QAction('Reset', self)
        self.resetButton.triggered.connect(self.Reset)
        self.optionMenu.addAction(self.resetButton)
        
        self.centerOfMass = QAction('Center of Mass', self)
        self.centerOfMass.setCheckable(True)
        self.centerOfMass.setChecked(False)
        self.optionMenu.addAction(self.centerOfMass)
        self.fftAct = QAction('Show FFT', self)
        self.fftAct.triggered.connect(self.showFFT)
        self.optionMenu.addAction(self.fftAct)

        # Ajouter l'action pour définir le nombre max de points
        self.setMaxPointsAct = QAction('Set Max Points...', self)
        self.setMaxPointsAct.triggered.connect(self.setMaxPointsDialog)
        self.optionMenu.addAction(self.setMaxPointsAct)
        
        # Ajouter l'action pour activer/désactiver les graphiques
        self.enablePlotsAct = QAction('Enable Plots Update', self)
        self.enablePlotsAct.setCheckable(True)
        self.enablePlotsAct.setChecked(True)  # Activé par défaut
        self.enablePlotsAct.triggered.connect(self.togglePlots)
        self.optionMenu.addAction(self.enablePlotsAct)
        
        # Ajouter l'action pour configurer la fréquence de mise à jour
        self.setUpdateRateAct = QAction('Set Update Rate...', self)
        self.setUpdateRateAct.triggered.connect(self.setUpdateRateDialog)
        self.optionMenu.addAction(self.setUpdateRateAct)
        self.setCrossToMeanAct = QAction('Set Cross to Mean Position', self)
        self.setCrossToMeanAct.setShortcut('Ctrl+Shift+M')
        self.setCrossToMeanAct.triggered.connect(self.setCrossToMean)
        self.optionMenu.addAction(self.setCrossToMeanAct)
        
        # Layout principal
        mainLayout = QVBoxLayout()
        mainLayout.setSpacing(10)
        mainLayout.setContentsMargins(10, 10, 10, 10)
        
        # ===== SECTION X =====
        xGroup = QGroupBox('X Position')
        xGroup.setStyleSheet("""
            QGroupBox {
                font: bold 12pt;
                color: #ff6b6b;
                border: 2px solid #ff6b6b;
                border-radius: 8px;
                margin-top: 8px;
                padding-top: 8px;
            }
            QGroupBox::title {
                subcontrol-origin: margin;
                left: 12px;
                padding: 0 5px 0 5px;
            }
        """)
        
        xLayout = QHBoxLayout()
        xLayout.setSpacing(8)
        
        # Graphique X 
        self.win3 = pg.PlotWidget()
        #self.win3.setDownsampling(mode='auto')
        #self.win3.setClipToView(True)
        self.win3.setBackground('#1e1e1e')
        self.win3.setMinimumHeight(300)
        
        self.p3 = self.win3.plot(pen=pg.mkPen('#ff6b6b', width=2),
                                 name="x",
                                 antialias=False,
                                 )
        self.win3.setLabel('left', 'X Position', color='#ff6b6b', **{'font-size': '11pt'})
        self.win3.setLabel('bottom', "Shot Number", color='white', **{'font-size': '11pt'})
        self.win3.showGrid(x=True, y=True, alpha=0.3)
        
        self.hLineMeanX = pg.InfiniteLine(angle=0, movable=False, 
                                          pen=pg.mkPen('#ff6b6b', width=2, style=QtCore.Qt.PenStyle.DashLine))
        self.win3.addItem(self.hLineMeanX, ignoreBounds=True)
        
        xLayout.addWidget(self.win3, stretch=5)  # Graph prend 5 parts
        
        # Statistiques X (cartes plus petites, verticales)
        xStatsLayout = QVBoxLayout()
        xStatsLayout.setSpacing(8)
        
        self.meanXCard = StatsCard('Mean', color='#ff6b6b')
        self.meanXCard.setMaximumWidth(100)
        self.stdXCard = StatsCard('Std', color='#ff6b6b')
        self.stdXCard.setMaximumWidth(100)
        self.pvXCard = StatsCard('P-V', color='#ff6b6b')
        self.pvXCard.setMaximumWidth(100)
        
        xStatsLayout.addWidget(self.meanXCard)
        xStatsLayout.addWidget(self.stdXCard)
        xStatsLayout.addWidget(self.pvXCard)
        xStatsLayout.addStretch()
        
        xLayout.addLayout(xStatsLayout, stretch=1)  # Stats prennent 1 part
        xGroup.setLayout(xLayout)
        mainLayout.addWidget(xGroup, stretch=1)
        
        # ===== SECTION Y =====
        yGroup = QGroupBox('Y Position')
        yGroup.setStyleSheet("""
            QGroupBox {
                font: bold 12pt;
                color: #51cf66;
                border: 2px solid #51cf66;
                border-radius: 8px;
                margin-top: 8px;
                padding-top: 8px;
            }
            QGroupBox::title {
                subcontrol-origin: margin;
                left: 12px;
                padding: 0 5px 0 5px;
            }
        """)
        
        yLayout = QHBoxLayout()
        yLayout.setSpacing(8)
        
        # Graphique Y (beaucoup plus grand)
        self.win4 = pg.PlotWidget()
        # self.win4.setDownsampling(mode='auto')
        #self.win4.setClipToView(True)
        self.win4.setBackground('#1e1e1e')
        self.win4.setMinimumHeight(300)
        
        self.p4 = self.win4.plot(pen=pg.mkPen('#51cf66', width=2),
                                 name="y",
                                 antialias=False,
                                 )
        self.win4.setLabel('left', 'Y Position', color='#51cf66', **{'font-size': '11pt'})
        self.win4.setLabel('bottom', "Shot Number", color='white', **{'font-size': '11pt'})
        self.win4.showGrid(x=True, y=True, alpha=0.3)

        self.hLineMeanY = pg.InfiniteLine(angle=0, movable=False, 
                                          pen=pg.mkPen('#51cf66', width=2, style=QtCore.Qt.PenStyle.DashLine))
        self.win4.addItem(self.hLineMeanY, ignoreBounds=True)
        
        yLayout.addWidget(self.win4, stretch=5)  # Graph prend 5 parts
        
        # Statistiques Y (cartes plus petites, verticales)
        yStatsLayout = QVBoxLayout()
        yStatsLayout.setSpacing(8)
        
        self.meanYCard = StatsCard('Mean', color='#51cf66')
        self.meanYCard.setMaximumWidth(100)
        self.stdYCard = StatsCard('Std', color='#51cf66')
        self.stdYCard.setMaximumWidth(100)
        self.pvYCard = StatsCard('P-V', color='#51cf66')
        self.pvYCard.setMaximumWidth(100)
        
        yStatsLayout.addWidget(self.meanYCard)
        yStatsLayout.addWidget(self.stdYCard)
        yStatsLayout.addWidget(self.pvYCard)
        yStatsLayout.addStretch()
        
        yLayout.addLayout(yStatsLayout, stretch=1)  # Stats prennent 1 part
        yGroup.setLayout(yLayout)
        mainLayout.addWidget(yGroup, stretch=1)
        
        # ===== SECTION INFO EN BAS =====
        infoLayout = QHBoxLayout()
        infoLayout.setSpacing(8)
        
        # Carte nombre de points (très petite)
        self.nbPointsCard = StatsCard('Buffer', color='#FFD700')
        self.nbPointsCard.setMaximumWidth(110)
        self.nbPointsCard.setMaximumHeight(70)
        
        # Carte taux de rafraîchissement (très petite)
        self.updateRateCard = StatsCard('Rate', color='#00CED1')
        self.updateRateCard.setValue(f'{1000/self.plot_update_interval:.0f}fps')
        self.updateRateCard.setMaximumWidth(90)
        self.updateRateCard.setMaximumHeight(70)
        
        infoLayout.addWidget(self.nbPointsCard)
        infoLayout.addWidget(self.updateRateCard)
        infoLayout.addStretch()
        
        mainLayout.addLayout(infoLayout)
        
        # Widget principal
        MainWidget = QWidget()
        MainWidget.setLayout(mainLayout)
        self.setCentralWidget(MainWidget)
        
        # Axes references
        self.axeY3 = self.win3.getAxis('left')
        self.axeY4 = self.win4.getAxis('left')
        # === MISE À JOUR DES LABELS D'AXES ===
        if self.current_stepX != 1:
            self.axeY3.setLabel('X(um) ' + self.current_label)
        else:
            self.axeY3.setLabel('X ' + self.current_label)
                
        if self.current_stepY != 1:
            self.axeY4.setLabel('Y(um) ' + self.current_label)
        else:
            self.axeY4.setLabel('Y ' + self.current_label)
        
        if self.parent is not None:
            self.parent.signalPointing.connect(self.Display)
    
    def setMaxPointsDialog(self):
        """Dialogue pour définir le nombre maximum de points"""
        value, ok = QInputDialog.getInt(
            self, 
            'Set Maximum Points', 
            f'Enter maximum number of points to keep in memory:\n(Current: {self.max_points})',
            value=self.max_points,
            min=100,
            max=100000,
            step=100
        )
        
        if ok:
            self.max_points = value  # Utilise le setter qui recrée les deques
            # Sauvegarder dans la config
            self.conf.setValue(self.name+"/max_points", self.max_points)
            self.conf.sync()
            
            # Mettre à jour l'affichage
            if len(self.Xec) > 0:
                self.updatePlots()  # ← Forcer la mise à jour
            
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Icon.Information)
            msg.setText(f"Maximum points set to {self.max_points}")
            msg.setInformativeText(f"Current number of points: {len(self.Xec)}")
            msg.setWindowTitle("Setting saved")
            msg.setWindowFlags(QtCore.Qt.WindowType.WindowStaysOnTopHint)
            msg.exec()
    
    def setUpdateRateDialog(self):
        """Dialogue pour définir la fréquence de mise à jour"""
        current_fps = 1000 / self.plot_update_interval
        
        value, ok = QInputDialog.getInt(
            self, 
            'Set Plot Update Rate', 
            f'Enter plot update rate (fps):\n(Current: {current_fps:.0f} fps = {self.plot_update_interval} ms)\n\nRecommendations:\n- 1-2 fps for high-speed acquisition (>100 Hz)\n- 5 fps for normal use\n- 10-20 fps for smooth display',
            value=int(current_fps),
            min=1,
            max=30,
            step=1
        )
        
        if ok:
            self.plot_update_interval = int(1000 / value)
            self.plot_update_timer.stop()
            self.plot_update_timer.start(self.plot_update_interval)
            self.updateRateCard.setValue(f'{value}fps')
            
            # Ajuster automatiquement update_every_n_points
            if value <= 2:
                self.update_every_n_points = 20  # Acquisition rapide
            elif value <= 5:
                self.update_every_n_points = 10
            else:
                self.update_every_n_points = 5
            
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Icon.Information)
            msg.setText(f"Plot update rate: {value} fps ({self.plot_update_interval} ms)")
            msg.setInformativeText(f"Updating every {self.update_every_n_points} points")
            msg.setWindowTitle("Setting saved")
            msg.setWindowFlags(QtCore.Qt.WindowType.WindowStaysOnTopHint)
            msg.exec()
    
    def togglePlots(self):
        """Activer/désactiver la mise à jour des graphiques"""
        if self.enablePlotsAct.isChecked():
            self.plot_update_timer.start(self.plot_update_interval)
            print('Plots update enabled')
        else:
            self.plot_update_timer.stop()
            print('Plots update disabled - data still recorded')
    
    def Display(self, obje):
        """Recevoir les données et les envoyer au worker"""
        # Mettre à jour le mode de calcul
        self.worker.use_center_of_mass = self.centerOfMass.isChecked()
        
        # Envoyer au worker (non-bloquant)
        self.worker.set_data(obje)
    
    def update_display(self, results):
        # print(f"update_display n={len(self.Xec)} t={time.time():.3f}")
        """Recevoir les résultats du worker - SANS mise à jour des widgets"""
        xec = results['xec']
        yec = results['yec']
        self.current_label = results['label']
        self.current_stepX = results['stepX']
        self.current_stepY = results['stepY']
        
        # Juste ajouter les données - PAS de mise à jour graphique ici !
        self.Xec.append(xec)
        self.Yec.append(yec)
        self.timestamps.append(time.time())
        # Démarrer le timer au premier point
        if not self.plot_update_timer.isActive():
            self.plot_update_timer.start(self.plot_update_interval)
        # Incrémenter le compteur
        self.update_counter += 1
        
        # Marquer qu'il y a de nouvelles données
        if self.update_counter >= self.update_every_n_points:
            self.new_data_available = True
            self.update_counter = 0
    
    def updatePlotsFromTimer(self):
        # self.time_old = time.time()
        """Mise à jour COMPLÈTE appelée par le timer uniquement"""
        # Ne rien faire s'il n'y a pas de nouvelles données
        if not self.new_data_available or len(self.Xec) == 0:
            return
        
        try:
            # Conversion UNE SEULE FOIS
            Xarray = np.array(self.Xec)
            Yarray = np.array(self.Yec)
            
            # === MISE À JOUR DES GRAPHIQUES ===
            
            n_points = len(Xarray)
            x_axis = np.arange(n_points)
            # Downsampling agressif si beaucoup de points
            
            self.p3.setData(x=x_axis, y=Xarray)
            self.p4.setData(x=x_axis, y=Yarray)
        
            # === CALCULS STATISTIQUES ===
            Xmean = Xarray.mean()
            Ymean = Yarray.mean()
            stdX = Xarray.std()
            stdY = Yarray.std()
            XPV = np.ptp(Xarray)
            YPV = np.ptp(Yarray)
            
            # === MISE À JOUR DES CARTES ===
            self.meanXCard.setValue(f'{Xmean:.2f}')
            self.stdXCard.setValue(f'{stdX:.2f}')
            self.pvXCard.setValue(f'{XPV:.2f}')
            
            self.meanYCard.setValue(f'{Ymean:.2f}')
            self.stdYCard.setValue(f'{stdY:.2f}')
            self.pvYCard.setValue(f'{YPV:.2f}')
            
            # === MISE À JOUR DES LIGNES MOYENNES ===
            self.hLineMeanX.setPos(Xmean)
            self.hLineMeanY.setPos(Ymean)
            
            # === MISE À JOUR DU BUFFER ===
            self.nbPointsCard.setValue(f"{n_points}/{self.max_points}")
            
        except Exception as e:
            print(f"Erreur dans updatePlotsFromTimer: {e}")
        finally:
            # Toujours réinitialiser le flag
            self.new_data_available = False
    
    def updatePlots(self):
        """Mettre à jour tous les plots - appelé manuellement (OpenF, Reset, etc.)"""
        if len(self.Xec) == 0:
            return
        
        # Forcer la mise à jour immédiate
        Xarray = np.array(self.Xec)
        Yarray = np.array(self.Yec)
        
        n_points = len(Xarray)
        x_axis = np.arange(n_points)
        self.p3.setData(x=x_axis, y=Xarray)
        self.p4.setData(x=x_axis, y=Yarray)
        
    def OpenF(self, fileOpen=False):
        if fileOpen is False:
            chemin = self.conf.value(self.name+"/path")
            fname = QFileDialog.getOpenFileName(self, "Open File", chemin, " 1D data (*.txt )")
            fichier = fname[0]
        else:
            fichier = str(fileOpen)
            
        if not fichier:
            return
            
        ext = os.path.splitext(fichier)[1]
        
        if ext == '.txt':
            aa = np.loadtxt(fichier, delimiter=" ", comments='#')
            self.Xec = deque(aa[0], maxlen=self.max_points)
            self.Yec = deque(aa[1], maxlen=self.max_points)
            
            # Forcer la mise à jour graphique
            self.updatePlots()  # ← Maintenant ça marche !
            
            # Mettre à jour les statistiques
            Xarray = np.array(self.Xec)
            Yarray = np.array(self.Yec)
            
            Xmean = Xarray.mean()
            Ymean = Yarray.mean()
            stdX = Xarray.std()
            stdY = Yarray.std()
            XPV = np.ptp(Xarray)
            YPV = np.ptp(Yarray)
            
            self.hLineMeanX.setPos(Xmean)
            self.hLineMeanY.setPos(Ymean)
            
            self.meanXCard.setValue(f'{Xmean:.2f}')
            self.stdXCard.setValue(f'{stdX:.2f}')
            self.pvXCard.setValue(f'{XPV:.2f}')
            
            self.meanYCard.setValue(f'{Ymean:.2f}')
            self.stdYCard.setValue(f'{stdY:.2f}')
            self.pvYCard.setValue(f'{YPV:.2f}')
            
            self.nbPointsCard.setValue(f"{len(self.Xec)}/{self.max_points}")
        else:
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Icon.Critical)
            msg.setText("Wrong file format !")
            msg.setInformativeText("The format of the file must be : .txt ")
            msg.setWindowTitle("Warning ...")
            msg.setWindowFlags(QtCore.Qt.WindowType.WindowStaysOnTopHint)
            msg.exec()
    
    def SaveF(self):
        
        fname = QFileDialog.getSaveFileName(self, "Save data as txt", self.path)
        if not fname[0]:
            return
            
        self.path = os.path.dirname(str(fname[0]))
        fichier = fname[0]
        print(fichier, ' is saved')
        self.conf.setValue(self.name+"/path", self.path)
        self.conf.sync()
        time.sleep(0.1)
        
        # Convertir deque en array pour sauvegarder
        a = np.array(self.Xec).reshape(-1, 1)
        b = np.array(self.Yec).reshape(-1, 1)
        mat = np.concatenate((a, b), axis=1)
        
        # Ajouter un header avec les infos
        header = f"Max points: {self.max_points}\nNumber of points: {len(self.Xec)}"
        np.savetxt(str(fichier)+'.txt', mat.T, header=header)
        
    def Reset(self):
        print('reset')
        
        # ARRÊTER le worker AVANT de vider les données
        self.worker.should_stop = True
        if self.worker.isRunning():
            self.worker.wait(200)  # Attendre max 200ms
        
        # Relancer le worker
        self.worker.should_stop = False
        self.worker.start() 
        
        # Arrêter le timer pendant le reset
        self.plot_update_timer.stop()
        
        # Vider les données
        self.Xec.clear()
        self.Yec.clear()
        
        # Effacer les graphiques
        self.p3.clear()
        self.p4.clear()
        
        
        # Réinitialiser les cartes
        self.nbPointsCard.setValue(f"0/{self.max_points}")
        self.meanXCard.setValue('--')
        self.stdXCard.setValue('--')
        self.pvXCard.setValue('--')
        self.meanYCard.setValue('--')
        self.stdYCard.setValue('--')
        self.pvYCard.setValue('--')
        
        # Réinitialiser les lignes moyennes
        self.hLineMeanX.setPos(0)
        self.hLineMeanY.setPos(0)
        
        # Réinitialiser les flags
        self.new_data_available = False
        self.update_counter = 0
        
        # Redémarrer le timer
        self.plot_update_timer.stop()
        
        print('Reset complete')

    def showFFT(self):
        """Ouvrir la fenêtre FFT avec les données actuelles"""
        if len(self.Xec) < 10:
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Icon.Warning)
            msg.setText("Not enough data points for FFT")
            msg.setInformativeText("Need at least 10 points")
            msg.setWindowTitle("Warning")
            msg.exec()
            return
        if len(self.timestamps) > 1:
            ts = np.array(self.timestamps)
            dt_measured = np.median(np.diff(ts))
            self.winFFT.dtInput.setText(f'{dt_measured:.4f}')
            
        self.winFFT.compute(np.array(self.Xec), np.array(self.Yec))
        self.winFFT.show()
        self.winFFT.raise_()

    def setCrossToMean(self):
        """Positionner la croix du parent sur la moyenne X, Y"""
        if len(self.Xec) == 0:
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Icon.Warning)
            msg.setText("No data available")
            msg.setInformativeText("Need at least one point to compute mean position")
            msg.setWindowTitle("Warning")
            msg.setWindowFlags(QtCore.Qt.WindowType.WindowStaysOnTopHint)
            msg.exec()
            return
        
        if self.parent is None:
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Icon.Warning)
            msg.setText("No parent window")
            msg.setInformativeText("This window needs a parent visu to set the cross position")
            msg.setWindowTitle("Warning")
            msg.setWindowFlags(QtCore.Qt.WindowType.WindowStaysOnTopHint)
            msg.exec()
            return
        
        # Calculer les moyennes
        Xmean = np.array(self.Xec).mean()
        Ymean = np.array(self.Yec).mean()
        
        # Convertir en pixels si nécessaire (les données sont en µm si stepX/Y != 1)
        if self.current_stepX != 1:
            xc_pixel = int(round(Xmean / self.current_stepX))
        else:
            xc_pixel = int(round(Xmean))
        
        if self.current_stepY != 1:
            yc_pixel = int(round(Ymean / self.current_stepY))
        else:
            yc_pixel = int(round(Ymean))
        
        # Mettre à jour la croix du parent
        try:
            self.parent.xc = xc_pixel
            self.parent.yc = yc_pixel
            self.parent.vLine.setPos(xc_pixel)
            self.parent.hLine.setPos(yc_pixel)
            
            # Mettre à jour le ROI si roiCross est activé
            if hasattr(self.parent, 'roiCross') and self.parent.roiCross:
                self.parent.ro1.setPos([xc_pixel - self.parent.rx/2, 
                                        yc_pixel - self.parent.ry/2])
            
            # Sauvegarder dans la config
            self.parent.conf.setValue(self.parent.name + "/xc", xc_pixel)
            self.parent.conf.setValue(self.parent.name + "/yc", yc_pixel)
            
            # Rafraîchir l'affichage
            self.parent.Coupe()
            
            print(f"Cross set to mean position: X={xc_pixel}, Y={yc_pixel}")
            
        except Exception as e:
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Icon.Critical)
            msg.setText("Error setting cross position")
            msg.setInformativeText(str(e))
            msg.setWindowTitle("Error")
            msg.setWindowFlags(QtCore.Qt.WindowType.WindowStaysOnTopHint)
            msg.exec()

    def closeEvent(self, event):
        """Arrêter le worker et le timer proprement"""
        print('Closing WINPOINTING...')
        
        # Arrêter le timer
        self.plot_update_timer.stop()
        
        # Arrêter le worker proprement
        self.worker.should_stop = True
        if self.worker.isRunning():
            print('Waiting for worker to finish...')
            self.worker.quit()
            self.worker.wait(500)  # Attendre max 500ms
            if self.worker.isRunning():
                print('Worker still running, terminating...')
                self.worker.terminate()
                self.worker.wait(200)
        
        self.isWinOpen = False
        self.E = []
        self.Xec.clear()
        self.Yec.clear()
        
        print('WINPOINTING closed')
        event.accept()


class WinPointingFFT(QMainWindow):
    """Fenêtre FFT séparée - calcul à la demande uniquement"""
    
    def __init__(self, parent=None):
        super().__init__()
        self.parent = parent
        self.isWinOpen = False
        self.setWindowTitle('FFT - Frequency Analysis')
        self.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
        
        if parent:
            self.setWindowIcon(QIcon(parent.icon + 'LOA.png'))
        
        self.setGeometry(150, 150, 1200, 700)
        self.Xdata = None
        self.Ydata = None
        self.setup()
    
    def setup(self):
        mainLayout = QVBoxLayout()
        mainLayout.setSpacing(8)
        mainLayout.setContentsMargins(10, 10, 10, 10)
        
        # === Barre d'options ===
        optionLayout = QHBoxLayout()
        
        dtLabel = QLabel('Δt between shots (s):')
        dtLabel.setStyleSheet("color: white; font: 10pt;")
        
        self.dtInput = QLineEdit('0.1')
        self.dtInput.setMaximumWidth(100)
        self.dtInput.setStyleSheet("font: 10pt;")
        self.dtInput.returnPressed.connect(self.recompute)
        
        self.recalcButton = QPushButton('Recompute')
        self.recalcButton.setStyleSheet("font: 10pt; padding: 4px 12px;")
        self.recalcButton.clicked.connect(self.recompute)
        
        optionLayout.addWidget(dtLabel)
        optionLayout.addWidget(self.dtInput)
        optionLayout.addWidget(self.recalcButton)
        optionLayout.addStretch()
        
        mainLayout.addLayout(optionLayout)
        
        # === Graphiques FFT ===
        plotsLayout = QHBoxLayout()
        
        # FFT X
        self.winFFTx = pg.PlotWidget(title='FFT X')
        self.winFFTx.setBackground('#1e1e1e')
        self.pFFTx = self.winFFTx.plot(pen=pg.mkPen('#ff6b6b', width=2))
        self.winFFTx.setLabel('left', 'Amplitude', color='#ff6b6b')
        self.winFFTx.setLabel('bottom', 'Frequency', color='white')
        self.winFFTx.showGrid(x=True, y=True, alpha=0.3)
        
        self.vLineFFTx = pg.InfiniteLine(angle=90, movable=False,
            pen=pg.mkPen('#ff6b6b', width=1, style=QtCore.Qt.PenStyle.DashLine))
        self.winFFTx.addItem(self.vLineFFTx, ignoreBounds=True)
        self.labelFFTx = pg.TextItem(color='#ff6b6b', anchor=(0, 1))
        self.winFFTx.addItem(self.labelFFTx)
        
        # FFT Y
        self.winFFTy = pg.PlotWidget(title='FFT Y')
        self.winFFTy.setBackground('#1e1e1e')
        self.pFFTy = self.winFFTy.plot(pen=pg.mkPen('#51cf66', width=2))
        self.winFFTy.setLabel('left', 'Amplitude', color='#51cf66')
        self.winFFTy.setLabel('bottom', 'Frequency', color='white')
        self.winFFTy.showGrid(x=True, y=True, alpha=0.3)
        
        self.vLineFFTy = pg.InfiniteLine(angle=90, movable=False,
            pen=pg.mkPen('#51cf66', width=1, style=QtCore.Qt.PenStyle.DashLine))
        self.winFFTy.addItem(self.vLineFFTy, ignoreBounds=True)
        self.labelFFTy = pg.TextItem(color='#51cf66', anchor=(0, 1))
        self.winFFTy.addItem(self.labelFFTy)
        
        plotsLayout.addWidget(self.winFFTx)
        plotsLayout.addWidget(self.winFFTy)
        mainLayout.addLayout(plotsLayout)
        
        # === Info en bas ===
        self.infoLabel = QLabel()
        self.infoLabel.setStyleSheet("color: #aaa; font: 10pt;")
        self.infoLabel.setAlignment(Qt.AlignmentFlag.AlignCenter)
        mainLayout.addWidget(self.infoLabel)
        
        widget = QWidget()
        widget.setLayout(mainLayout)
        self.setCentralWidget(widget)
    
    def compute(self, Xarray, Yarray):
        """Stocker les données et calculer la FFT"""
        self.Xdata = Xarray.copy()
        self.Ydata = Yarray.copy()
        self._doFFT()
    
    def recompute(self):
        """Recalculer avec le nouveau dt"""
        if self.Xdata is not None:
            self._doFFT()
    
    def _doFFT(self):
        """Calcul FFT interne"""
        n = len(self.Xdata)
        if n < 10:
            return
        
        # Lire dt
        try:
            dt = float(self.dtInput.text())
            if dt <= 0:
                dt = 1.0
        except ValueError:
            dt = 1.0
        
        # Soustraire la moyenne
        Xc = self.Xdata - self.Xdata.mean()
        Yc = self.Ydata - self.Ydata.mean()
        
        # Fenêtre de Hanning
        window = np.hanning(n)
        Xc = Xc * window
        Yc = Yc * window
        
        # FFT
        fftX = np.abs(np.fft.rfft(Xc)) * 2 / n
        fftY = np.abs(np.fft.rfft(Yc)) * 2 / n
        
        # Fréquences en Hz
        freq = np.fft.rfftfreq(n, d=dt)
        freq_unit = 'Hz'
        period_unit = 's'
        
        # Ignorer DC
        fftX[0] = 0
        fftY[0] = 0
        
        # Afficher
        self.pFFTx.setData(x=freq, y=fftX)
        self.pFFTy.setData(x=freq, y=fftY)
        
        self.winFFTx.setLabel('bottom', f'Frequency ({freq_unit})', color='white')
        self.winFFTy.setLabel('bottom', f'Frequency ({freq_unit})', color='white')
        
        # Pic dominant X
        if len(fftX) > 1:
            idx = np.argmax(fftX[1:]) + 1
            f0 = freq[idx]
            T = 1.0 / f0 if f0 > 0 else float('inf')
            self.vLineFFTx.setPos(f0)
            self.labelFFTx.setText(f'f = {f0:.2f} {freq_unit}  (T = {T:.3f} {period_unit})')
            self.labelFFTx.setPos(f0, fftX[idx])
        
        # Pic dominant Y
        if len(fftY) > 1:
            idx = np.argmax(fftY[1:]) + 1
            f0 = freq[idx]
            T = 1.0 / f0 if f0 > 0 else float('inf')
            self.vLineFFTy.setPos(f0)
            self.labelFFTy.setText(f'f = {f0:.2f} {freq_unit}  (T = {T:.3f} {period_unit})')
            self.labelFFTy.setPos(f0, fftY[idx])
        
        # Info
        nyquist = 1.0 / (2 * dt)
        df = 1.0 / (n * dt)
        self.infoLabel.setText(
            f'{n} points | Δt = {dt} s ({1/dt:.1f} Hz rep. rate) | '
            f'Nyquist = {nyquist:.1f} {freq_unit} | '
            f'Resolution = {df:.4f} {freq_unit}'
        )
    
    def closeEvent(self, event):
        self.isWinOpen = False
        event.accept()



if __name__ == "__main__":
    appli = QApplication(sys.argv)
    appli.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
    e = WINPOINTING(name='VISU')
    e.show()
    appli.exec()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import numpy as np 
import pathlib
import sys
from PyQt6.QtWidgets import QApplication, QWidget, QLineEdit
from PyQt6.QtWidgets import QVBoxLayout, QHBoxLayout,QMessageBox
from PyQt6.QtWidgets import QLabel, QSizePolicy, QCheckBox, QPushButton
from PyQt6.QtWidgets import QFileDialog, QProgressBar, QDoubleSpinBox
from PyQt6.QtWidgets import QMainWindow, QStatusBar, QSpacerItem
from PyQt6.QtGui import QAction
from PyQt6 import QtCore, QtGui
from PyQt6.QtCore import Qt
from PyQt6.QtGui import QIcon

import time
import os
import qdarkstyle
from scipy.integrate import solve_ivp
from scipy.interpolate import interp1d, RectBivariateSpline
import matplotlib.pyplot as plt
import pyqtgraph as pg
from scipy.constants import c, m_e, e
from visu import WinCut

# La coordonnee y decrit l'axe de l'aimant et x decrit la coordonnee transverse
# (0,0) correspond au centre de l'aimant.
# y>0: vers le lanex.
# Oxyz repere oriente dans le sens trigonometrique. Le champ magnetique B est suivant z.
# La position du zero du Lanex  corresponde a la position du pixel dans l'input Electrons du spectro 


class WINRESULT(QWidget):
    
    def __init__(self, parent=None):
        super().__init__()
        self.parent = parent
        self.isWinOpen = False
        self.setup()
        self.actionButton()
    
    def setup(self):
        vbox = QHBoxLayout()
        lablePos = QLabel('Position on lanex')
        self.posBox = QDoubleSpinBox()
        self.posBox.setSuffix(' mm')
        self.posBox.setRange(-10000, 100000)
        lableE = QLabel('E')
        self.EBox = QDoubleSpinBox()
        self.EBox.setSuffix(' Mev')
        self.EBox.setRange(0, 1000000)
        vbox.addWidget(lablePos)
        vbox.addWidget(self.posBox)
        vbox.addWidget(lableE)
        vbox.addWidget(self.EBox)
        self.setWindowTitle('Energy calculation')
        self.setLayout(vbox)
    
    def actionButton(self):
        self.posBox.editingFinished.connect(self.calculE)

    def calculE(self):
        # Find the Enrgy for a postion on the lanex
        f_E = interp1d(self.parent.threadCalcul.s, self.parent.threadCalcul.E)
        if self.posBox.value() < self.parent.threadCalcul.s.max():
            self.EBox.setValue(f_E(self.posBox.value()))
        
    def closeEvent(self, event):
        """ when closing the window
        """
        self.isWinOpen = False

class WINTRAJECTOIRE(QMainWindow):
    signalPlot = QtCore.pyqtSignal(object)

    def __init__(self, parent=None, **kwds):

        super().__init__()
        self.parent = parent
        self.isWinOpen = False
        self.p = pathlib.Path(__file__)
        sepa = os.sep
        self.icon = str(self.p.parent) + sepa+'icons' + sepa
        self.paraFile = QtCore.QSettings(str(self.p.parent / 'defaultTrajectory.ini'), QtCore.QSettings.Format.IniFormat)
        self.winresult = WINRESULT(parent=self)
        self.setup()
        self.iniValue()
        self.threadCalcul = CALCULTHREAD(parent=self)
        self.actionButton()
        self.CheckChangeB()
        
    def setup(self):
        # define button
        self.setWindowTitle('Electron Trajectory ')
        self.setWindowIcon(QIcon(self.icon+'LOA.png'))
        menubar = self.menuBar()
        menubar.setNativeMenuBar(False)
        self.fileMenu = menubar.addMenu('&File')
        self.ResultMenu = menubar.addMenu('&Results')
        self.AboutMenu = menubar.addMenu('&About')
        self.statusBar = QStatusBar()
        self.setContentsMargins(0, 0, 0, 0)
        self.aboutAction = QAction(QtGui.QIcon(self.icon+"LOA.png'"),
                                   'Geometry', self)
        
        self.AboutMenu.addAction(self.aboutAction)
        self.aboutAction.triggered.connect(self.aboutFunc)

        self.setStatusBar(self.statusBar)
        self.openAct = QAction(QtGui.QIcon(self.icon+"Open.png"),
                               'Open parameter File', self)

        self.openAct.setShortcut('Ctrl+o')
        self.openAct.triggered.connect(self.openF)
        self.fileMenu.addAction(self.openAct)

        self.saveAct = QAction(QtGui.QIcon(self.icon+"Saving.png"),
                               'Save Parameter', self)

        self.saveAct.setShortcut('Ctrl+s')
        self.saveAct.triggered.connect(self.saveF)
        self.fileMenu.addAction(self.saveAct)

        self.exportAct = QAction(QtGui.QIcon(self.icon+"Settings.png"),
                                 'Export dispersion', self)
        self.exportAct.triggered.connect(self.exportF)
        self.fileMenu.addAction(self.exportAct)

        self.resAct = QAction('Energy cal', self)
        self.resAct.triggered.connect(lambda: self.open_widget(self.winresult))
        self.ResultMenu.addAction(self.resAct)

        self.winDisp = WinCut.GRAPHCUT(self, title='Plot Dispersion' )
        self.DispAct = QAction('Plot  Dispersion', self)
        self.DispAct.triggered.connect(lambda: self.open_widget(self.winDisp))
        self.DispAct.triggered.connect(self.DispDisplay)
        self.ResultMenu.addAction(self.DispAct)

        self.winDist = WinCut.GRAPHCUT(self, title='Plot Electron length path' )
        self.DistAct = QAction('Plot  Distance Electron length path', self)
        self.DistAct.triggered.connect(lambda: self.open_widget(self.winDist))
        self.DistAct.triggered.connect(self.DistDisplay)
        self.ResultMenu.addAction(self.DistAct)

        self.winDsDE = WinCut.GRAPHCUT(self, title='Plot Ds/DE')
        self.DsDEAct = QAction('Plot  ds/dE', self)
        self.DsDEAct.triggered.connect(lambda: self.open_widget(self.winDsDE))
        self.DsDEAct.triggered.connect(self.DsDEDisplay)
        self.ResultMenu.addAction(self.DsDEAct)

        self.winResol = WinCut.GRAPHCUT(self, title='Resolution')
        self.ResolAct = QAction('Plot  Resolution', self)
        self.ResolAct.triggered.connect(lambda: self.open_widget(self.winResol))
        self.ResolAct.triggered.connect(self.ResolDisplay)
        self.ResultMenu.addAction(self.ResolAct)

        vbox1 = QVBoxLayout()

        Hbox1 = QHBoxLayout()
        labelSource = QLabel('Position Source ')
        labelSource.setAlignment(Qt.AlignmentFlag.AlignCenter)
        labelSourceX = QLabel('X')
        self.sourceX = QDoubleSpinBox()
        self.sourceX.setSuffix('mm')
        self.sourceX.setRange(-100000, 10000)
        labelSourceY = QLabel('Y')
        self.sourceY = QDoubleSpinBox()
        self.sourceY.setSuffix('mm')
        self.sourceY.setRange(-100000, 10000)
        Hbox1.addWidget(labelSource)
        Hbox1.addWidget(labelSourceX)
        Hbox1.addWidget(self.sourceX)
        Hbox1.addWidget(labelSourceY)
        Hbox1.addWidget(self.sourceY)
        divLabel = QLabel('div')
        Hbox1.addWidget(divLabel)
        self.divBox = QDoubleSpinBox()
        self.divBox.setSuffix(' mrad')
        Hbox1.addWidget(self.divBox)
        labelAngle = QLabel('Angle Faisceau/oy')
        self.angle = QDoubleSpinBox()
        self.angle.setSuffix(' °')
        self.angle.setRange(-180, 180)
        Hbox1.addWidget(labelAngle)
        Hbox1.addWidget(self.angle)
        Spacer1 = QSpacerItem(1, 1, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
        Hbox1.addItem(Spacer1)
        vbox1.addLayout(Hbox1)

        Hbox3 = QHBoxLayout()
        labelLanex = QLabel('Position Lanex')
        labelLanexX = QLabel('X')
        self.lanexX = QDoubleSpinBox()
        self.lanexX.setSuffix(' mm')
        self.lanexX.setRange(-10000, 10000)
        Hbox3.addWidget(labelLanex)
        Hbox3.addWidget(labelLanexX)
        Hbox3.addWidget(self.lanexX)
        labelLanexY = QLabel('Y')
        self.lanexY = QDoubleSpinBox()
        self.lanexY.setSuffix(' mm')
        self.lanexY.setRange(-10000, 10000)
        Hbox3.addWidget(labelLanexY)
        Hbox3.addWidget(self.lanexY)

        labelLanexAngle = QLabel('Angle(°)/ox')
        self.lanexAngle = QDoubleSpinBox()
        self.lanexAngle.setSuffix(' °')
        self.lanexAngle.setRange(-360, 360)
        Hbox3.addWidget(labelLanexAngle)
        Hbox3.addWidget(self.lanexAngle)
        Spacer3 = QSpacerItem(1, 1, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
        Hbox3.addItem(Spacer3)
        vbox1.addLayout(Hbox3)

        Hbox4 = QHBoxLayout()
        labelB = QLabel('B')
        self.Bbox = QDoubleSpinBox()
        self.Bbox.setSuffix(' T')
        labelSens = QLabel('B sens')
        self.BSensBox = QDoubleSpinBox()
        self.BSensBox.setRange(-1, 1)
        self.checkB = QCheckBox('B constant')

        self.checkB.setChecked(True)
        self.labelAimantLength = QLabel('Magnet length')
        self.AimantLength = QDoubleSpinBox()
        self.AimantLength.setSuffix(" mm")
        self.AimantLength.setRange(0, 10000)
        self.labelAimantWidth = QLabel('Magnet width')
        self.AimantWidth = QDoubleSpinBox()
        self.AimantWidth.setSuffix(" mm")
        self.AimantWidth.setRange(0, 10000)

        Hbox4.addWidget(labelB)
        Hbox4.addWidget(self.Bbox)
        Hbox4.addWidget(labelSens)
        Hbox4.addWidget(self.BSensBox)
        Hbox4.addWidget(self.checkB)
        Hbox4.addWidget(self.labelAimantLength)
        Hbox4.addWidget(self.AimantLength)
        Hbox4.addWidget(self.labelAimantWidth)
        Hbox4.addWidget(self.AimantWidth)
        Spacer4 = QSpacerItem(1, 1, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
        Hbox4.addItem(Spacer4)
        vbox1.addLayout(Hbox4)
        Hbox4b = QHBoxLayout()
        self.BButton = QPushButton('Select B file', self)
        Hbox4b.addWidget(self.BButton)
        self.pathBLine = QLineEdit(self)
        Hbox4b.addWidget(self.pathBLine)
        vbox1.addLayout(Hbox4b)

        Hbox5 = QHBoxLayout()
        paraLabel = QLabel('Parameters')
        Hbox5.addWidget(paraLabel)
        vbox1.addLayout(Hbox5)

        Hbox6 = QHBoxLayout()
        EminLabel = QLabel('Emin')
        Hbox6.addWidget(EminLabel)
        self.EminBox = QDoubleSpinBox()
        self.EminBox.setSuffix(' Mev')
        self.EminBox.setRange(0, 10000)
        Hbox6.addWidget(self.EminBox)

        EmaxLabel = QLabel('Emax')
        Hbox6.addWidget(EmaxLabel)
        self.EmaxBox = QDoubleSpinBox()
        self.EmaxBox.setRange(self.EminBox.value(), 100000)
        self.EmaxBox.setSuffix(' Mev')
        Hbox6.addWidget(self.EmaxBox)
        
        NLabel = QLabel('Number traj')
        Hbox6.addWidget(NLabel)
        self.NBox = QDoubleSpinBox()
        self.NBox.setRange(1, 10000)
        Hbox6.addWidget(self.NBox)

        lanexLabel = QLabel('Lanex length')
        Hbox6.addWidget(lanexLabel)
        self.lanexBox = QDoubleSpinBox()
        self.lanexBox.setSuffix(' mm')
        self.lanexBox.setRange(0, 10000)
        Hbox6.addWidget(self.lanexBox)
        Spacer6 = QSpacerItem(1, 1, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
        Hbox6.addItem(Spacer6)
        vbox1.addLayout(Hbox6)
        
        Hbox7 = QHBoxLayout()
        self.calculButton = QPushButton('Start Calcul', self)
        Hbox7.addWidget(self.calculButton)
        self.stopButton = QPushButton('STOP', self)
        Hbox7.addWidget(self.stopButton)
        self.progressBar = QProgressBar()
        Hbox7.addWidget(self.progressBar)
        self.EcalLabel = QLabel('')
        Hbox7.addWidget(self.EcalLabel)
        vbox1.addLayout(Hbox7)

        self.winImage = pg.GraphicsLayoutWidget()
        self.winImage.setContentsMargins(0, 0, 0, 0)
        self.winImage.setAspectLocked(False)
        self.winImage.ci.setContentsMargins(0, 0, 0, 0)
        vbox1.addWidget(self.winImage)
        vbox1.setContentsMargins(0, 0, 0, 0)
        
        self.p1 = self.winImage.addPlot()
        self.imh = pg.ImageItem()
        self.axeX = self.p1.getAxis('bottom')
        self.axeY = self.p1.getAxis('left')
        self.p1.addItem(self.imh)
        self.p1.setMouseEnabled(x=False, y=False)
        self.p1.setContentsMargins(0, 0, 0, 10)
        
        self.winImage2 = pg.GraphicsLayoutWidget()
        self.winImage2.setAspectLocked(True)
        # vbox1.addWidget(self.winImage2) # affiche le plot de la dispersion sous les trejectoires
        self.p2 = self.winImage2.addPlot()

        self.p1.showAxis('right', show=False)
        self.p1.showAxis('top', show=False)
        self.p1.showAxis('left', show=True)
        self.p1.setLabel('left', " Y Axis (mm)")
        self.p1.setLabel('bottom', "X Axis(mm)")
        self.p2.setLabel('left', "Lanex (mm)")
        self.p2.setLabel('bottom', "Energy (Mev) ")
        self.p2.setTitle('Dispersion on lanex ')

        self.label_Cross = QLabel()
        self.label_Cross.setStyleSheet("font:12pt")
        self.statusBar.addPermanentWidget(self.label_Cross)

        MainWidget = QWidget()
        MainWidget.setLayout(vbox1)
        self.setCentralWidget(MainWidget)
        self.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))

    def actionButton(self):

        self.proxy = pg.SignalProxy(self.p1.scene().sigMouseMoved, rateLimit=60, slot=self.mouseMoved)
        self.vb = self.p1.vb
        self.sourceX.editingFinished.connect(self.set_default)
        self.sourceY.editingFinished.connect(self.set_default)
        self.angle.editingFinished.connect(self.set_default)
        self.lanexX.editingFinished.connect(self.set_default)
        self.lanexY.editingFinished.connect(self.set_default)
        self.lanexAngle.editingFinished.connect(self.set_default)
        self.Bbox.editingFinished.connect(self.set_default)
        self.BSensBox.editingFinished.connect(self.set_default)
        self.EminBox.editingFinished.connect(self.set_default)
        self.EmaxBox.editingFinished.connect(self.set_default)
        self.NBox.editingFinished.connect(self.set_default)
        self.divBox.editingFinished.connect(self.set_default)
        self.lanexBox.editingFinished.connect(self.set_default)
        self.AimantLength.editingFinished.connect(self.set_default)
        self.AimantWidth.editingFinished.connect(self.set_default)
        self.BButton.clicked.connect(self.BLoad)
        self.calculButton.clicked.connect(self.startCalcul)
        self.stopButton.clicked.connect(self.stopCalcul)
        self.threadCalcul.remain.connect(self.progress)
        self.checkB.stateChanged.connect(self.CheckChangeB)

    def iniValue(self):
        # set value from ini file 
        self.sourceX.setValue(float(self.paraFile.value("/"+"/x_j")))
        self.sourceY.setValue(float(self.paraFile.value("/"+"/y_j")))
        self.angle.setValue(float(self.paraFile.value("/"+"/theta_e")))
        self.lanexAngle.setValue(float(self.paraFile.value("/"+"/theta_la")))
        self.Bbox.setValue(float(self.paraFile.value("/"+"/B")))
        self.lanexX.setValue(float(self.paraFile.value("/"+"/x_d")))
        self.lanexY.setValue(float(self.paraFile.value("/"+"/y_d")))
        self.EminBox.setValue(float(self.paraFile.value("/"+"/Emin")))
        self.EmaxBox.setValue(float(self.paraFile.value("/"+"/Emax", )))
        self.NBox.setValue(float(self.paraFile.value("/"+"/N")))
        self.divBox.setValue(float(self.paraFile.value("/"+"/div")))
        self.lanexBox.setValue(float(self.paraFile.value("/"+"/long_lanex")))
        self.BSensBox.setValue(float(self.paraFile.value("/"+"/sens_B")))
        self.pathBLine.setText(self.paraFile.value("/"+"/Bpath"))
        self.BFileName = self.paraFile.value("/"+"/Bpath")
        self.AimantLength.setValue(float(self.paraFile.value("/"+"/LM")))
        self.AimantWidth.setValue(float(self.paraFile.value("/"+"/LaM")))
        self.set_default()

    def set_default(self):
        # save values in the default ini file and read value 
        self.paraFile.setValue("/"+"/x_j", self.sourceX.value())
        self.paraFile.setValue("/"+"/y_j", self.sourceY.value())
        self.paraFile.setValue("/"+"/theta_e", self.angle.value())
        self.paraFile.setValue("/"+"/theta_la", self.lanexAngle.value())
        self.paraFile.setValue("/"+"/B", self.Bbox.value())
        self.paraFile.setValue("/"+"/x_d", self.lanexX.value())
        self.paraFile.setValue("/"+"/y_d", self.lanexY.value())
        self.paraFile.setValue("/"+"/Emin", self.EminBox.value())
        self.paraFile.setValue("/"+"/Emax", self.EmaxBox.value())
        self.paraFile.setValue("/"+"/N", self.NBox.value())
        self.paraFile.setValue("/"+"/div", self.divBox.value())
        self.paraFile.setValue("/"+"/long_lanex", self.lanexBox.value())
        self.paraFile.setValue("/"+"/sens_B", self.BSensBox.value())
        self.paraFile.setValue("/"+"/LM", self.AimantLength.value())
        self.paraFile.setValue("/"+"/LaM", self.AimantWidth.value())
        
        # # Position du jet par rapport ? (0,0):
        self.x_j = self.sourceX.value()  # 0  # en mm
        self.y_j = self.sourceY.value()  # -1200  # en mm
        
        # # Angle du faisceau par rapport a l'axe y
        self.theta_e = self.angle.value()*np.pi/180  # 0

        # # Position Lanex
        self.x_d = self.lanexX.value()  # 250
        self.y_d = self.lanexY.value()  # 675
        self.theta_la = self.lanexAngle.value()  # -90  #
        self.theta_l = self.theta_la*np.pi/180

        # # Champ
        self.B = self.Bbox.value()  # 1.4  # Tesla
        self.L = self.AimantLength.value()
        self.La = self.AimantWidth.value()

        # # Parametres de Calcul
        self.Emin = int(self.EminBox.value())  # 20  # en Mev
        self.Emax = int(self.EmaxBox.value())  # 4000
        self.Nt = int(self.NBox.value())  # 100  # nombre de trjectoire
        self.progressBar.setMaximum(int(self.Nt)-1)
        self.div = self.divBox.value() * 1e-3  # 1e-3  # divergence FWHM du faisceau en radian (calcul resolution) 
        self.long_lanex = self.lanexBox.value()  # 870  # mm
        self.sens_B = self.BSensBox.value()  # -1  # =1 si le fichier contenant Bz(x,y) donne le bon signe, =-1 inverse le signe de B(x,y) 

    def BLoad(self):
        dialog = QFileDialog()
        self.BFileName, _ = dialog.getOpenFileName()
        # Update the label text to display the selected file name
        if self.BFileName:
            self.pathBLine.setText(str(self.BFileName))
            self.paraFile.setValue("/"+"/Bpath", self.BFileName)

    def startCalcul(self):
        self.threadCalcul.start()

    def progress(self, rem):
        self.progressBar.setValue(int(rem[0]))
        self.EcalLabel.setText(str(round(rem[1],2)) + ' Mev')
        self.EcalLabel.setStyleSheet("QLabel {color:rgb(%f,%f,%f)}" % (rem[2][0], rem[2][1], rem[2][2]))

    def stopCalcul(self):
        self.threadCalcul.stop = True

    def openF(self):
        # open ini saved file
        fname = QFileDialog.getOpenFileNames(self, "Open File")
        self.openedFiles = fname[0]
        self.openedFiles = fname[0]
        fichier = self.openedFiles[0]
        ext = os.path.splitext(fichier)[1]
        print('fichier to load', fichier)
        if ext == '.ini':  # ini fi
            self.paraFile = QtCore.QSettings(fichier, QtCore.QSettings.Format.IniFormat)
            self.iniValue()

    def saveF(self):
        # save ini saved file
        print(open)
        fname = QFileDialog.getSaveFileName(self, "Save parameter file ")
        self.path = os.path.dirname(str(fname[0]))
        fichier = fname[0]+'.ini'
        print(fichier, ' is saved')
        fichierS = str(self.p.parent / 'defaultTrajectory.ini')
        with open(fichierS, 'r') as f:
            dataS = f.read()
        with open(fichier, 'w') as f:
            f.write(dataS)

    def exportF(self):
        # open ini saved file
        # 1ere colonne = energie electron en MeV
        # 2eme colonne = derivee de s par l'energie E (ds/dE) en mm/MeV
        # 3eme colonne = position s sur le lanex en mm
        # 4eme colonne = angle des electrons par rapport ? Ox (sens)
        # 5eme colone = distance parcourue par les electrons
        fname = QFileDialog.getSaveFileName(self, "export dispersion file ")
        fichier = fname[0]+'.txt'
        dat = np.array([self.threadCalcul.EInter, self.threadCalcul.ds_dE, self.threadCalcul.sInter, self.threadCalcul.thetaInter, self.threadCalcul.DsourceInter])
        dat = dat.T
        np.savetxt(str(fichier), dat)
        if self.parent is not None:
            self.parent.confSpectro.setValue("/"+"/dsde_name",str(fichier))
    
    def mouseMoved(self, evt):
        # to print position of the mouse on the second graph   
        pos = evt[0]  # using signal proxy turns original arguments into a tuple
        
        if self.threadCalcul.E is not None:
            if self.p1.sceneBoundingRect().contains(pos):
                mousePoint = self.vb.mapSceneToView(pos)
                mx = np.array([abs(i-mousePoint.x()) for i in self.threadCalcul.E])
                index = mx.argmin()
                #if index >= 0 and index < len(self.threadCalcul.E):
                self.xMouse = (mousePoint.x())
                self.yMouse = (mousePoint.y())
                    # self.xMc = self.threadCalcul.E[index]
                    # self.yMc = self.threadCalcul.s[index]
                self.label_Cross.setText(str(round((self.xMouse), 2)) + '    ,    '+ str(round(self.yMouse, 2)))
                    
    def CheckChangeB(self):
        if self.checkB.isChecked() is True:
            self.AimantLength.setEnabled(True)
            self.AimantWidth.setEnabled(True)
            self.labelAimantLength.setEnabled(True)
            self.labelAimantWidth.setEnabled(True)
            self.BButton.setEnabled(False)
            self.pathBLine.setEnabled(False)
        else:
            self.AimantLength.setEnabled(False)
            self.AimantWidth.setEnabled(False)
            self.labelAimantLength.setEnabled(False)
            self.labelAimantWidth.setEnabled(False)
            self.BButton.setEnabled(True)
            self.pathBLine.setEnabled(True)

    def DispDisplay(self):
        self.winDisp.PLOT(self.threadCalcul.sInter, axis=self.threadCalcul.EInter, label='E (Mev)', labelY='Lanex Screen (mm)')

    def DistDisplay(self):
        self.winDist.PLOT(self.threadCalcul.DsourceInter, axis=self.threadCalcul.EInter, label='E (Mev)', labelY='D (mm)')
    
    def DsDEDisplay(self):
        self.winDsDE.PLOT(self.threadCalcul.ds_dE, axis=self.threadCalcul.EInter, label='E (Mev)', labelY='Ds/DE (mm/Mev)')

    def ResolDisplay(self):
        self.winResol.PLOT(self.threadCalcul.resolInter, axis=self.threadCalcul.EInter, label='E (Mev)', labelY='Resolution(%)')

    def open_widget(self, fene):
        """ open new widget
        """
        
        if fene.isWinOpen is False:
            fene.setup
            fene.isWinOpen = True
            fene.show()
        else:
            # fene.activateWindow()
            # fene.raise_()
            fene.showNormal()

    def aboutFunc(self):
        dlg = QMessageBox(self)
        dlg.setWindowTitle("Geometry definition")
        dlg.setText(" (0,0) is the magnet center \n Vertical y>0 : to the lanex screen\n X is horizontal axe \n B = B ez\n zero lanex is pixel chosen in electron input ")
        dlg.exec()
        print('ok')
    def closeEvent(self, event):
        """ when closing the window
        """
        self.isWinOpen = False

class CALCULTHREAD(QtCore.QThread):
    '''
    Secon thread  to calcul the trajectory
    '''
    remain = QtCore.pyqtSignal(object)
    

    def __init__(self, parent=None):
        super(CALCULTHREAD, self).__init__()
        self.parent = parent
        self.E = None
        self.aa = 0
        

    def wc_interp(self, x, y):
        #  return wc interpollated x,y in mm =0 if x,y  out of the file
        #  f  = interp2d(self.xmap, self.ymap, self.wcmap)  # depreciated
        f = RectBivariateSpline(self.xmap, self.ymap, self.wcmap.T)
        if x < self.xmap.min() or x > self.xmap.max() or y < self.ymap.min() or y > self.ymap.max():
            # x,y not in the B file wc=0
            return 0
        else:
            return f(x, y)[0][0]

    def odefun(self, t, y):
        # differential equation
        # v=Y[0]ex+y[1]ey x=Y[2]ex+Y[3]ey
        dydt = np.zeros(4)  
        if self.parent.checkB.isChecked() is False:  # b from file with interpolation

            dydt[0] = - self.wc_interp(y[2]*1e3, y[3]*1e3) * y[1] / np.sqrt(1 + y[0]**2 + y[1]**2)
            dydt[1] = self.wc_interp(y[2]*1e3, y[3]*1e3) * y[0] / np.sqrt(1 + y[0]**2 + y[1]**2)
            dydt[2] = c * y[0] / np.sqrt(1 + y[0]**2 + y[1]**2)
            dydt[3] = c * y[1] / np.sqrt(1 + y[0]**2 + y[1]**2)

        else:  # B constant
            
            if (-1*self.parent.L/2  < y[3]*1e3 < self.parent.L/2 ) and (-self.parent.La/2 < y[2]*1e3  < self.parent.La/2):
                #print('dans aimant')
                dydt[0] = -self.wc * y[1] / np.sqrt(1 + y[0]**2 + y[1]**2)
                dydt[1] = self.wc * y[0] / np.sqrt(1 + y[0]**2 + y[1]**2)
                dydt[2] = c * y[0] / np.sqrt(1 + y[0]**2 + y[1]**2)
                dydt[3] = c * y[1] / np.sqrt(1 + y[0]**2 + y[1]**2)
            else:
                # print('hors aimant')resol
                dydt[0] = 0  # B=0
                dydt[1] = 0
                dydt[2] = c * y[0] / np.sqrt(1 + y[0]**2 + y[1]**2)
                dydt[3] = c * y[1] / np.sqrt(1 + y[0]**2 + y[1]**2)
        return dydt

    def run(self):
        # when start button is pressed start the calculation 
        print('calcul ...')
        self.parent.p1.clear()
        self.parent.p2.clear()
        self.stop = False
        self.parent.p1.addItem(self.parent.imh)
        time.sleep(0.1)
        E = []
        sol = []

        def event(t, x):
            # to stop the calculation when electron arrive on the lanex 
            D = (x[2] * 1e3 - self.parent.x_d)  * np.tan(self.parent.theta_l) - (x[3] * 1e3 - self.parent.y_d)  # intersection dans le plan du lanex
            #D = (x[2] * 1e3 - self.parent.x_d)
            return D

        event.terminal = True

        # # Energie de electrons :resol
        E = np.linspace(self.parent.Emin, self.parent.Emax, self.parent.Nt)
        dE = abs(E[2]-E[1])

        # Gamma
        gamma = E * (e/(m_e * c**2) * 1e6) + 1  # E est en MeV  
        beta = np.sqrt((gamma**2)-1)/gamma

        self.wc = self.parent.sens_B*e * self.parent.B/m_e  # wc est la fréquence cyclotron

        # Positions initiales 
        x0 = np.zeros(self.parent.Nt) + self.parent.x_j*1e-3
        y0 = np.zeros(self.parent.Nt) + self.parent.y_j*1e-3

        # Impulsions initiales (ici u=gamma*beta=p/mc)
        ux0 = -np.ones(self.parent.Nt) * gamma * beta * np.sin(self.parent.theta_e)
        uy0 = np.ones(self.parent.Nt) * gamma * beta * np.cos(self.parent.theta_e)

        if self.parent.checkB.isChecked() is False:
            data_B = np.loadtxt(str(self.parent.BFileName))
            self.xmap = data_B[0, 1:]
            print('bMax',max(self.xmap))
            self.xmap = data_B[0, 1:] 
            self.xmax = max(self.xmap)
            self.xmin = min(self.xmap)
            self.ymap = data_B[1:, 0]
            self.ymax = max(self.ymap)
            self.ymin = min(self.ymap)
            self.Bmap = data_B[1:, 1:]
            print('B Max',self.Bmap.max())
            if self.xmap[0]>self.xmap[1] :
                self.xmap = np.flip(self.xmap)
            if self.ymap[0]>self.ymap[1] :
                self.ymap = np.flip(self.ymap)    
            self.wcmap = self.parent.sens_B*e*self.Bmap/m_e
            print('calul avec B file',self.xmax,self.xmin,self.ymax,self.ymin)
            print(self.xmap)
            #  Translation and scaling the image
            tr = QtGui.QTransform()  # prepare ImageItem transformation:
            self.scaleX = self.xmap[1] - self.xmap[2]       
            self.scaleY = self.ymap[1] - self.ymap[2]
            tr.scale(self.scaleX, self.scaleY)  # scale horizontal and vertical axes
            self.transX = (self.xmax-self.xmin)/2
            self.transY = (self.ymax-self.ymin)/2
            tr.translate(self.transX/self.scaleX, self.transY/self.scaleY)  # to locate maximum at axis origin      
            self.parent.imh.setImage(self.Bmap.T)
            self.parent.imh.setTransform(tr)  # assign transform
            # plt.imshow(self.Bmap)
            # plt.show()
        else:
            #  plot rectangle to show  the magnet
            self.parent.p1.plot([-self.parent.La/2, self.parent.La/2], [-self.parent.L/2, -self.parent.L/2], pen='b')
            self.parent.p1.plot([-self.parent.La/2, self.parent.La/2], [self.parent.L/2, self.parent.L/2], pen='b')
            self.parent.p1.plot([-self.parent.La/2, -self.parent.La/2], [-self.parent.L/2, self.parent.L/2], pen='b')
            self.parent.p1.plot([self.parent.La/2, self.parent.La/2], [-self.parent.L/2, self.parent.L/2], pen='b')
        
        #  =============================================================================
        #  Projection sur le lanex
        #  =============================================================================

        x_lanex_end = self.parent.x_d - self.parent.long_lanex * np.cos(self.parent.theta_l)
        y_lanex_end = self.parent.y_d - self.parent.long_lanex * np.sin(self.parent.theta_l)
        #  plot line to show the lanex
        self.parent.p1.plot([self.parent.x_d, x_lanex_end], [self.parent.y_d, y_lanex_end], pen='red')
        
        Dsource = np.zeros(self.parent.Nt)
        x_P = np.zeros(self.parent.Nt)
        y_P = np.zeros(self.parent.Nt)
        theta = np.zeros(self.parent.Nt)
        #calcul trajectoire
        for i in range(0, self.parent.Nt):
            if self.stop is True:
                break
            # print('trajectoire nb : ', i, E[i], 'Mev')
            #input('appuyer pour continuer...')
        
            # Résolution de l'équation différentielle
            
            sol = solve_ivp(self.odefun, [0, abs(3000*np.pi/self.wc)], 
                            np.array([ux0[i], uy0[i], x0[i], y0[i]]),
                            method='LSODA', events=event,rtol=3e-14) #'LSODA' RK45 atol=3e-14, 
            
            # Représentation graphique des trajectoires
            # define random color
            col = (255*np.random.random(), 255*np.random.random(), 255*np.random.random())
            prog = [i, E[i], col] # send to mainGui i, E,color for the progress bar and show Energy 
            self.remain.emit(prog) 
            if i % 5 ==0 :  # too slow with a lot of trejectory :plot only the 5th and then only one
                self.winplot = self.parent.p1.plot(sol.y[2] * 1e3, sol.y[3] * 1e3, pen=col, clear=False)
            else:
                self.winplot.setData(sol.y[2] * 1e3, sol.y[3] * 1e3, pen=col)
            # a=[sol.y[2], sol.y[3]]
            # self.plotData.emit(a)

            # Calcul de la distance parcourue par les électrons
            for j in range(0, (sol.y[1]).shape[0]-1):
                Dsource[i] = Dsource[i] + np.sqrt((sol.y[2, j+1] * 1e3 - sol.y[2, j] * 1e3)**2 + (sol.y[3, j+1] * 1e3 - sol.y[3,j] * 1e3)**2)
            
            # Résultats
            x_P[i] = sol.y[2, -1] * 1e3
            y_P[i] = sol.y[3, -1] * 1e3    # Point P l'arrivée sur le lanex (coordonnées en mm)
            theta[i] = np.arctan2(sol.y[1, -1], sol.y[0, -1])       # angle de sortie de l'électron par rapport à l'axe Ox (sens trigonométrique)
            #print(E[i], 'Mev pos lanex : ', x_P[i], y_P[i], theta[i],'rad')
        
        # Coordonnée s le long du détecteur
        if np.cos(self.parent.theta_l) < 0.1:
            s = -(y_P - self.parent.y_d) / np.sin(self.parent.theta_l)
        else:
            s = -(x_P - self.parent.x_d) / np.cos(self.parent.theta_l)

        #print('Coordonees le long lanex', s)

        # Calcul de ds_dE 
        self.E = E
        self.EInter = E[1:] - dE / 2
        self.s = s  # position sur le dectecteur
        self.theta = theta
        self.ds_dE = np.diff(self.s) / np.diff(E)
        
        self.Dsource = Dsource  # distance parcouru
        
        f_s3 = interp1d(E, s) # fct for  interpol 
        self.sInter = f_s3(self.EInter) # intef pol de s vs E
        f_theta3 = interp1d(E, theta)
        self.thetaInter = f_theta3(self.EInter)

        f_d = interp1d(E, Dsource) # fct for  interpol 
        self.DsourceInter = f_d(self.EInter)
        # plot dispersion en fct E 
        #self.parent.p2.plot(E, self.s, symbol='+', name='no interp', symbolBrush='w', symbolSize= 3)
        #self.parent.p2.plot(self.EInter, self.sInter, pen='r', name="interpolation")
        
        self.resolInter = 2*self.DsourceInter*np.tan(self.parent.div/2)

        self.resolInter= 100*(self.resolInter/self.ds_dE )/ self.EInter

        # self.parent.p2.plot(self.E, self.ds_dE3)
        # self.parent.p2.plot(self.E, self.ds_dE, pen='r')

        # self.parent.p2.plot(self.E, self.Dsource, symbol='t')
        # self.parent.p2.plot(self.EInter, self.DsourceInter, pen='b')


if __name__ == "__main__":

    appli = QApplication(sys.argv)
    #appli.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
    w = WINTRAJECTOIRE()
    w.show()
    appli.exec()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Window for Measurement
Réécriture avec connexion ZMQ directe au serveur RSAI
Sans utiliser le client MOTORRSAI complet

@author: juliengautier
@modified: 2025 - Connexion ZMQ directe
"""

import qdarkstyle
from PyQt6 import QtCore, QtGui
from PyQt6.QtWidgets import (QApplication, QVBoxLayout, QHBoxLayout, QMainWindow, 
                              QHeaderView, QWidget, QTableWidget, QTableWidgetItem, 
                              QAbstractItemView, QComboBox, QInputDialog, QLabel,
                              QDialog, QLineEdit, QDialogButtonBox, QFormLayout,
                              QMessageBox, QGroupBox)
from PyQt6.QtGui import QAction, QIcon
from PyQt6.QtCore import Qt, QMutex,QTimer
from visu.WinCut import GRAPHCUT
from visu.winZoom import ZOOM
import pathlib
import numpy as np
import sys
import time
import os
import zmq
import ast
from scipy import ndimage


class ServerConfigDialog(QDialog):
    """
    Dialogue pour configurer l'adresse IP et le port du serveur ZMQ
    """
    
    def __init__(self, current_host='localhost', current_port='5555', parent=None):
        super().__init__(parent)
        self.setWindowTitle("Configuration Serveur ZMQ")
        self.setModal(True)
        self.setMinimumWidth(350)
        
        # Appliquer le style sombre
        self.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
        
        layout = QVBoxLayout()
        
        # Groupe de configuration
        groupBox = QGroupBox("Paramètres du serveur RSAI")
        formLayout = QFormLayout()
        
        # Champ IP/Host
        self.hostEdit = QLineEdit(current_host)
        self.hostEdit.setPlaceholderText("Ex: 192.168.1.100 ou localhost")
        formLayout.addRow("Adresse IP / Host:", self.hostEdit)
        
        # Champ Port
        self.portEdit = QLineEdit(current_port)
        self.portEdit.setPlaceholderText("Ex: 5555")
        formLayout.addRow("Port:", self.portEdit)
        
        groupBox.setLayout(formLayout)
        layout.addWidget(groupBox)
        
        # Info
        infoLabel = QLabel("⚠️ Les modifications seront appliquées au prochain démarrage.")
        infoLabel.setStyleSheet("color: orange; font-style: italic;")
        layout.addWidget(infoLabel)
        
        # Boutons
        buttonBox = QDialogButtonBox(
            QDialogButtonBox.StandardButton.Ok | 
            QDialogButtonBox.StandardButton.Cancel
        )
        buttonBox.accepted.connect(self.accept)
        buttonBox.rejected.connect(self.reject)
        layout.addWidget(buttonBox)
        
        self.setLayout(layout)
    
    def getValues(self):
        """Retourne les valeurs saisies"""
        return self.hostEdit.text().strip(), self.portEdit.text().strip()


class ZMQMotorClient:
    """
    Client ZMQ léger pour communiquer avec le serveur RSAI
    Basé sur le code fonctionnel de mainMotor.py
    """
    
    def __init__(self, server_host='localhost', server_port='5555'):
        self.server_address = f"tcp://{server_host}:{server_port}"
        self.context = zmq.Context()
        self.socket = None
        self.isconnected = False
        self.server_available = False
        self.mut = QMutex()
        self._connect()
    
    def _connect(self):
        """Établit la connexion ZMQ DEALER"""
        try:
            if self.socket:
                self.socket.close()
            
            self.socket = self.context.socket(zmq.DEALER)
            self.socket.setsockopt(zmq.RCVTIMEO, 5000)  # 5s timeout
            self.socket.setsockopt(zmq.SNDTIMEO, 5000)
            self.socket.setsockopt(zmq.LINGER, 1000)
            self.socket.setsockopt(zmq.SNDHWM, 100)
            self.socket.setsockopt(zmq.RCVHWM, 100)
            
            # Identité unique
            import uuid
            identity = f"MEAS_{uuid.uuid4()}".encode('utf-8')
            self.socket.setsockopt(zmq.IDENTITY, identity)
            
            self.socket.connect(self.server_address)
            
            # Vider le buffer au démarrage (comme dans mainMotor.py)
            self.socket.setsockopt(zmq.LINGER, 0)
            self.socket.setsockopt(zmq.RCVTIMEO, 1000)  # Timeout 1 seconde
            
            # Vider les messages résiduels éventuels
            try:
                while True:
                    self.socket.recv(zmq.NOBLOCK)
                    print("🧹 Message résiduel vidé")
            except zmq.Again:
                print("✅ Buffer vidé, socket prêt")
            
            # Remettre le timeout normal
            self.socket.setsockopt(zmq.RCVTIMEO, 5000)
            
            self.isconnected = True
            self.server_available = True
            print(f"✅ ZMQMotorClient connecté à {self.server_address}")
            time.sleep(0.5)  # Attendre un peu comme dans mainMotor.py
            
        except Exception as e:
            self.isconnected = False
            self.server_available = False
            print(f"❌ Erreur connexion ZMQ: {e}")
    
    def sendMessage(self, message):
        """Envoie un message et retourne la réponse"""
        if not self.server_available:
            print("⚠️ Serveur non disponible")
            return "error: server not available"
        
        self.mut.lock()
        try:
            # DEALER envoie: [frame vide, message]
            self.socket.send(b'', zmq.SNDMORE)
            self.socket.send_string(message)
            
            # Recevoir: frame vide + réponse
            empty = self.socket.recv()
            retour_brut = self.socket.recv_string()
            
            # Nettoyer la réponse
            response = retour_brut.strip()
            
            self.isconnected = True
            self.server_available = True
            return response
            
        except zmq.Again:
            print(f"⏱️ Timeout ZMQ pour: {message}")
            self.isconnected = False
            self.server_available = False
            return "error: timeout"
        except Exception as e:
            print(f"❌ Erreur ZMQ: {e}")
            self.isconnected = False
            self.server_available = False
            return f"error: {e}"
        finally:
            self.mut.unlock()
    
    def getListRack(self):
        """Récupère la liste des IPs des racks"""
        response = self.sendMessage("listRack")
        print(f"📋 getListRack response: '{response}'")
        try:
            result = ast.literal_eval(response)
            print(f"📋 Liste des racks: {result}")
            return result
        except Exception as e:
            print(f"❌ Erreur parsing listRack: {e}")
            return []
    
    def getDict(self):
        """Récupère le dictionnaire des moteurs"""
        response = self.sendMessage("dict")
        print(f"📋 getDict response length: {len(response)}")
        try:
            result = ast.literal_eval(response)
            return result
        except Exception as e:
            print(f"❌ Erreur parsing dict: {e}")
            return {}
    
    def getNbMotorRack(self):
        """Récupère le nombre de moteurs par rack"""
        response = self.sendMessage("nbMotRack")
        print(f"📋 getNbMotorRack response: '{response}'")
        try:
            return ast.literal_eval(response)
        except:
            return []
    
    def getRackName(self, ip):
        """Récupère le nom du rack"""
        response = self.sendMessage(f"{ip}, 1, nomRack")
        return response if response else ip
    
    def getMotorList(self, ip, dict_moteurs):
        """Récupère la liste des noms de moteurs pour un rack"""
        dict_name = f"self.dictMotor_{ip}"
        print(f"🔍 Recherche moteurs pour: {dict_name}")
        print(f"🔍 Clés disponibles: {list(dict_moteurs.keys())}")
        
        if dict_name in dict_moteurs:
            motor_dict = dict_moteurs[dict_name]
            motor_names = []
            # Trier les clés numériques
            numeric_keys = sorted([k for k in motor_dict.keys() if isinstance(k, int)])
            for key in numeric_keys:
                # Récupérer le nom du moteur
                name = self.sendMessage(f"{ip}, {key}, name")
                motor_names.append(name if name else f"Motor_{key}")
            print(f"📋 Moteurs trouvés: {motor_names}")
            return motor_names
        return []
    
    def getMotorListFromCount(self, ip, count):
        """Récupère la liste des noms de moteurs en utilisant le nombre de moteurs"""
        motor_names = []
        for i in range(1, count + 1):
            name = self.sendMessage(f"{ip}, {i}, name")
            motor_names.append(name if name else f"Motor_{i}")
        return motor_names
    
    def getPosition(self, ip, numMotor):
        """Récupère la position du moteur"""
        response = self.sendMessage(f"{ip}, {numMotor}, position")
        try:
            return float(response)
        except:
            return 0.0
    
    def getStep(self, ip, numMotor):
        """Récupère la valeur du pas"""
        response = self.sendMessage(f"{ip}, {numMotor}, step")
        try:
            return float(response)
        except:
            return 1.0
    
    def getMotorName(self, ip, numMotor):
        """Récupère le nom du moteur"""
        return self.sendMessage(f"{ip}, {numMotor}, name")
    
    def close(self):
        """Ferme la connexion"""
        try:
            if self.socket:
                self.socket.close()
            self.context.term()
        except:
            pass
        self.isconnected = False


class MEAS(QMainWindow):
    
    signalPlot = QtCore.pyqtSignal(object)
    
    def __init__(self, parent=None, conf=None, name='VISU', confMot=None, **kwds):
        
        super().__init__()

        self.parent = parent
        self.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
        p = pathlib.Path(__file__)
        sepa = os.sep
        
        if conf is None:
            self.conf = QtCore.QSettings(str(p.parent / 'confVisu.ini'), QtCore.QSettings.Format.IniFormat)
        else:
            self.conf = conf
        self.confMot = confMot
        self.name = name
        self.ThresholdState = False
        self.symbol = False
        
        # Configuration serveur ZMQ
        self.serverHost = 'localhost'
        self.serverPort = '5555'
        
        # Lecture config serveur si disponible
        fileconf = str(p.parent) + sepa + "confServer.ini"
        if os.path.exists(fileconf):
            confServer = QtCore.QSettings(fileconf, QtCore.QSettings.Format.IniFormat)
            self.serverHost = str(confServer.value('MAIN/server_host', 'localhost'))
            self.serverPort = str(confServer.value('MAIN/serverPort', '5555'))
        
        # Gestion moteurs RSAI via ZMQ
        self.motRSAI = kwds.get('motRSAI', False)
        self.zmqClient = None
        self.listRack = []
        self.listRackNames = []
        self.listMotorName = []
        self.dict_moteurs = {}
        self.currentIP = None
        self.currentMotorNum = 0
        self.stepmotor = 1
        
        if self.motRSAI:
            self._initZMQConnection()
        
        # Gestion moteurs A2V (inchangé)
        self.motA2V = kwds.get('motA2V', False)
        if self.motA2V:
            sys.path.append('C:/Users/UPX/Desktop/python/camera')
            import moteurA2V as A2V
            self.A2V = A2V
            self.listMotorName = self.A2V.listMotorName
            self.listMotor = self.A2V.listMotor
            print('TMCL motor connected to database')

        self.indexUnit = 1  # micron
        self.icon = str(p.parent) + sepa + 'icons' + sepa
        self.isWinOpen = False
        self.setup()
        
        self.setWindowTitle('MEASUREMENTS')
        self.shoot = 0
        self.nomFichier = ' '
        self.TableSauv = ['file,Max,Min,x Max,y max,Sum,Mean,Size,x c.mass,y c.mass,user1']
        
        self.path = self.conf.value(self.name + "/path")
        
        # Création des fenêtres de graphiques
        self.winCoupeMax = GRAPHCUT(parent=self, conf=self.conf, name=self.name, symbol='t', pen=None, lastColored=True)
        self.winCoupeMin = GRAPHCUT(parent=self, conf=self.conf, name=self.name, symbol='t', pen=None, lastColored=True)
        self.winCoupeXmax = GRAPHCUT(parent=self, conf=self.conf, name=self.name, symbol='t', pen=None, lastColored=True)
        self.winCoupeYmax = GRAPHCUT(parent=self, conf=self.conf, name=self.name, symbol='t', pen=None, lastColored=True)
        self.winCoupeSum = GRAPHCUT(parent=self, conf=self.conf, name=self.name, symbol='t', pen=None, lastColored=True)
        self.winCoupeMean = GRAPHCUT(parent=self, conf=self.conf, name=self.name, symbol='t', pen=None, lastColored=True)
        self.winCoupeXcmass = GRAPHCUT(parent=self, conf=self.conf, name=self.name, symbol='t', pen=None, lastColored=True)
        self.winCoupeYcmass = GRAPHCUT(parent=self, conf=self.conf, name=self.name, symbol='t', pen=None, lastColored=True)
        self.winCoupeSumThreshold = GRAPHCUT(parent=self, conf=self.conf, name=self.name, symbol='t', pen=None, lastColored=True)
        self.winCoupeUser1 = GRAPHCUT(parent=self, conf=self.conf, name=self.name, symbol='t', pen=None, lastColored=True)
        self.signalTrans = dict()
        
        # Listes de données
        self.Maxx = []
        self.Minn = []
        self.Summ = []
        self.Mean = []
        self.Xmax = []
        self.Ymax = []
        self.Xcmass = []
        self.Ycmass = []
        self.labelsVert = []
        self.posMotor = []
        self.SummThre = []
        self.USER1 = []
        
        # Fenêtres zoom
        self.winZoomMax = ZOOM()
        self.winZoomSum = ZOOM()
        self.winZoomMean = ZOOM()
        self.winZoomXmax = ZOOM()
        self.winZoomYmax = ZOOM()
        self.winZoomCxmax = ZOOM()
        self.winZoomCymax = ZOOM()
        self.winZoomSumThreshold = ZOOM()
        self.winZoomUser1 = ZOOM()
        
        self.maxx = 0
        self.summ = 0
        self.moy = 0
        self.user1 = 0
        self.label = 'Shoot'
        self.setWindowIcon(QIcon(self.icon + 'LOA.png'))
        self.setGeometry(100, 300, 1000, 300)
        
        if self.motRSAI or self.motA2V:
            self.unit()
            
            # Timer pour mise à jour de la position moteur (200ms)
            self.positionTimer = QTimer()
            self.positionTimer.timeout.connect(self._updatePositionLabel)
            self.positionTimer.start(200)

    def _initZMQConnection(self):
        """Initialise la connexion ZMQ et récupère les infos des racks/moteurs"""
        try:
            self.zmqClient = ZMQMotorClient(self.serverHost, self.serverPort)
            
            if self.zmqClient.isconnected:
                # Récupérer la liste des racks
                self.listRack = self.zmqClient.getListRack()
                print(f"🔧 Racks trouvés: {self.listRack}")
                
                if not self.listRack:
                    print("⚠️ Aucun rack trouvé, vérifiez le serveur")
                    return
                
                # Récupérer le nombre de moteurs par rack
                self.nbMotorRack = self.zmqClient.getNbMotorRack()
                print(f"🔧 Nombre de moteurs par rack: {self.nbMotorRack}")
                
                # Récupérer les noms des racks
                self.listRackNames = []
                for ip in self.listRack:
                    name = self.zmqClient.getRackName(ip)
                    self.listRackNames.append(name)
                print(f"🔧 Noms des racks: {self.listRackNames}")
                
                # Récupérer le dictionnaire des moteurs
                self.dict_moteurs = self.zmqClient.getDict()
                print(f"🔧 Dict moteurs récupéré: {len(self.dict_moteurs)} entrées")
                
                # Initialiser avec le premier rack
                if self.listRack:
                    self.currentIP = self.listRack[0]
                    self._updateMotorList()
                
                print('✅ RSAI motors connected via ZMQ server')
            else:
                print('⚠️ Impossible de se connecter au serveur ZMQ')
                self.motRSAI = False
                
        except Exception as e:
            print(f'❌ Erreur initialisation ZMQ: {e}')
            import traceback
            traceback.print_exc()
            self.motRSAI = False

    def _updateMotorList(self):
        """Met à jour la liste des moteurs pour le rack actuel"""
        if self.currentIP and self.zmqClient:
            # Essayer d'abord avec le dictionnaire
            self.listMotorName = self.zmqClient.getMotorList(self.currentIP, self.dict_moteurs)
            
            # Si pas de résultat, utiliser le nombre de moteurs
            if not self.listMotorName and hasattr(self, 'nbMotorRack') and self.nbMotorRack:
                try:
                    idx = self.listRack.index(self.currentIP)
                    count = self.nbMotorRack[idx]
                    print(f"🔧 Utilisation du nombre de moteurs: {count}")
                    self.listMotorName = self.zmqClient.getMotorListFromCount(self.currentIP, count)
                except Exception as e:
                    print(f"❌ Erreur récupération moteurs: {e}")
            
            print(f"🔧 Liste moteurs pour {self.currentIP}: {self.listMotorName}")

    def setFile(self, file):
        self.nomFichier = file
        
    def setup(self):
        vLayout = QVBoxLayout()
        hLayout1 = QHBoxLayout()
        
        menubar = self.menuBar()
        menubar.setNativeMenuBar(False)
        self.fileMenu = menubar.addMenu('&File')
        self.PlotMenu = menubar.addMenu('&Plot')
        self.ZoomMenu = menubar.addMenu('&Zoom')
        self.settingsMenu = menubar.addMenu('&Settings')
        
        self.ThresholdAct = QAction('Threshold', self)
        self.ThresholdMenu = menubar.addAction(self.ThresholdAct)
        self.ThresholdAct.triggered.connect(self.Threshold)
        
        self.setContentsMargins(0, 0, 0, 0)
       
        self.openAct = QAction(QtGui.QIcon(self.icon + "Open.png"), 'Open File', self)
        self.openAct.setShortcut('Ctrl+o')
        self.openAct.triggered.connect(self.openF)
        self.fileMenu.addAction(self.openAct)
        
        self.saveAct = QAction(QtGui.QIcon(self.icon + "disketteSave.png"), 'Save file', self)
        self.saveAct.setShortcut('Ctrl+s')
        self.saveAct.triggered.connect(self.saveF)
        self.fileMenu.addAction(self.saveAct)
        
        # Menu Settings
        self.serverConfigAct = QAction('🔧 Configure Serveur Motors ZMQ...', self)
        self.serverConfigAct.triggered.connect(self.openServerConfig)
        self.settingsMenu.addAction(self.serverConfigAct)
        
        self.reconnectAct = QAction('🔄 Reconnect', self)
        self.reconnectAct.triggered.connect(self.reconnectServer)
        self.settingsMenu.addAction(self.reconnectAct)
        
        self.settingsMenu.addSeparator()
        
        self.showConnectionInfoAct = QAction('ℹ️ Info Connexion', self)
        self.showConnectionInfoAct.triggered.connect(self.showConnectionInfo)
        self.settingsMenu.addAction(self.showConnectionInfoAct)
        
        # Menus Plot
        self.PlotMenu.addAction('max', self.PlotMAX)
        self.PlotMenu.addAction('min', self.PlotMIN)
        self.PlotMenu.addAction('x max', self.PlotXMAX)
        self.PlotMenu.addAction('y max', self.PlotYMAX)
        self.PlotMenu.addAction('Sum', self.PlotSUM)
        self.PlotMenu.addAction('Mean', self.PlotMEAN)
        self.PlotMenu.addAction('x center mass', self.PlotXCMASS)
        self.PlotMenu.addAction('y center mass', self.PlotYCMASS)
        self.PlotMenu.addAction('User 1', self.PlotUSER1)

        # Menus Zoom
        self.ZoomMenu.addAction('max', self.ZoomMAX)
        self.ZoomMenu.addAction('Sum', self.ZoomSUM)
        self.ZoomMenu.addAction('Mean', self.ZoomMEAN)
        self.ZoomMenu.addAction('X max', self.ZoomXmax) 
        self.ZoomMenu.addAction('Y max', self.ZoomYmax)
        self.ZoomMenu.addAction(' X c.of.m', self.ZoomCxmax)
        self.ZoomMenu.addAction(' Y c.of.m', self.ZoomCymax)
        self.ZoomMenu.addAction(' User1', self.ZoomUser1)
        
        self.But_reset = QAction('Reset', self)
        menubar.addAction(self.But_reset)
        
        hLayout2 = QHBoxLayout()
        self.table = QTableWidget()
        hLayout2.addWidget(self.table)
        
        self.table.setColumnCount(12)
        self.table.setHorizontalHeaderLabels(('File', 'Max', 'Min', 'x max', 'y max', 'Sum', 'Mean', 'Size', 'x c.mass', 'y c.mass', 'user1', 'date'))
        header = self.table.horizontalHeader()
        header.setDefaultAlignment(Qt.AlignmentFlag.AlignHCenter)
        header.setDefaultAlignment(Qt.AlignmentFlag.AlignVCenter)
        header.setSectionResizeMode(QHeaderView.ResizeMode.ResizeToContents)

        # Configuration pour moteurs RSAI ou A2V
        if self.motRSAI or self.motA2V:
            # Indicateur de connexion
            self.connectionLabel = QLabel()
            self._updateConnectionStatus()
            hLayout1.addWidget(self.connectionLabel)
            
            # Sélecteur d'unité
            self.unitBouton = QComboBox()
            self.unitBouton.addItems(['Step', 'µm', 'mm', 'ps', '°'])
            self.unitBouton.setMaximumWidth(100)
            self.unitBouton.setMinimumWidth(80)
            self.unitBouton.setCurrentIndex(self.indexUnit)
            self.unitBouton.currentIndexChanged.connect(self.unit)
            hLayout1.addWidget(self.unitBouton)
            
            # Sélecteur de rack (IP)
            self.rackChoise = QComboBox()
            self.rackChoise.setMinimumWidth(200)
            if self.motRSAI:
                for i, ip in enumerate(self.listRack):
                    name = self.listRackNames[i] if i < len(self.listRackNames) else ip
                    self.rackChoise.addItem(f"{name}  ({ip})")
            self.rackChoise.currentIndexChanged.connect(self.ChangeIPRack)
            hLayout1.addWidget(self.rackChoise)
            
            # Sélecteur de moteur
            self.motorNameBox = QComboBox()
            self.motorNameBox.setMinimumWidth(200)
            self.motorNameBox.addItem('Choose a Motor')
            if self.motRSAI:
                self.motorNameBox.addItems(self.listMotorName)
            elif self.motA2V:
                self.motorNameBox.addItems(self.listMotorName)
            self.motorNameBox.currentIndexChanged.connect(self.motorChange)
            hLayout1.addWidget(self.motorNameBox)
            
            # Label position moteur
            self.positionLabel = QLabel("Pos: ---")
            self.positionLabel.setStyleSheet("font-weight: bold; color: #00ff00;")
            self.positionLabel.setMinimumWidth(150)
            hLayout1.addWidget(self.positionLabel)
            
            # Configuration table avec colonne Motor
            self.table.setColumnCount(13)
            self.table.setHorizontalHeaderLabels(('File', 'Max', 'Min', 'x max', 'y max', 'Sum', 'Mean', 'Size', 'x c.mass', 'y c.mass', 'user1', 'Motor', 'date'))
             
        self.table.horizontalHeader().setVisible(True)
        self.table.setAlternatingRowColors(True)
        self.table.resizeColumnsToContents()
        self.table.setEditTriggers(QAbstractItemView.EditTrigger.NoEditTriggers)
        
        vLayout.addLayout(hLayout1)
        vLayout.addLayout(hLayout2)
        
        MainWidget = QWidget()
        MainWidget.setLayout(vLayout)
        self.setCentralWidget(MainWidget)
        self.setContentsMargins(1, 1, 1, 1)
        
        if self.parent is not None:
            self.parent.signalMeas.connect(self.Display)
            
        self.But_reset.triggered.connect(self.Reset)

    def _updateConnectionStatus(self):
        """Met à jour l'indicateur de connexion"""
        if hasattr(self, 'connectionLabel'):
            if self.zmqClient and self.zmqClient.isconnected:
                self.connectionLabel.setText("🟢 Motors Connected")
                self.connectionLabel.setStyleSheet("color: #00ff00; font-weight: bold;")
            else:
                self.connectionLabel.setText("🔴 Motors not Connected")
                self.connectionLabel.setStyleSheet("color: #ff0000; font-weight: bold;")

    def _updatePositionLabel(self):
        """Met à jour l'affichage de la position du moteur"""
        if hasattr(self, 'positionLabel') and self.motRSAI:
            if self.motorNameBox.currentIndex() > 0 and self.zmqClient:
                pos = self.zmqClient.getPosition(self.currentIP, self.currentMotorNum)
                pos_converted = pos / self.unitChange if self.unitChange != 0 else pos
                self.positionLabel.setText(f"Pos: {pos_converted:.2f} {self.unitName}")
            else:
                self.positionLabel.setText("Pos: ---")

    def openServerConfig(self):
        """Ouvre le dialogue de configuration du serveur"""
        dialog = ServerConfigDialog(
            current_host=self.serverHost,
            current_port=self.serverPort,
            parent=self
        )
        
        if dialog.exec() == QDialog.DialogCode.Accepted:
            new_host, new_port = dialog.getValues()
            
            # Validation basique
            if not new_host:
                QMessageBox.warning(self, "Erreur", "L'adresse IP ne peut pas être vide.")
                return
            
            if not new_port.isdigit() or not (1 <= int(new_port) <= 65535):
                QMessageBox.warning(self, "Erreur", "Le port doit être un nombre entre 1 et 65535.")
                return
            
            # Sauvegarder dans le fichier ini
            self._saveServerConfig(new_host, new_port)
            
            # Demander si on veut reconnecter maintenant
            reply = QMessageBox.question(
                self, 
                "Reconnecter ?",
                f"Configuration sauvegardée:\n\nHost: {new_host}\nPort: {new_port}\n\nVoulez-vous reconnecter maintenant ?",
                QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No,
                QMessageBox.StandardButton.Yes
            )
            
            if reply == QMessageBox.StandardButton.Yes:
                self.serverHost = new_host
                self.serverPort = new_port
                self.reconnectServer()
    
    def _saveServerConfig(self, host, port):
        """Sauvegarde la configuration du serveur dans confServer.ini"""
        p = pathlib.Path(__file__)
        sepa = os.sep
        fileconf = str(p.parent) + sepa + "confServer.ini"
        
        confServer = QtCore.QSettings(fileconf, QtCore.QSettings.Format.IniFormat)
        confServer.setValue('MAIN/server_host', host)
        confServer.setValue('MAIN/serverPort', port)
        confServer.sync()
        
        print(f"✅ Configuration sauvegardée: {host}:{port}")
    
    def reconnectServer(self):
        """Reconnecte au serveur ZMQ"""
        print(f"🔄 Reconnexion au serveur {self.serverHost}:{self.serverPort}...")
        
        # Fermer l'ancienne connexion
        if self.zmqClient:
            self.zmqClient.close()
            time.sleep(0.3)
        
        # Réinitialiser les données
        self.listRack = []
        self.listRackNames = []
        self.listMotorName = []
        self.dict_moteurs = {}
        self.currentIP = None
        
        # Reconnecter
        self._initZMQConnection()
        
        # Mettre à jour l'interface
        self._updateConnectionStatus()
        
        if self.motRSAI and hasattr(self, 'rackChoise'):
            # Mettre à jour le combo des racks
            self.rackChoise.clear()
            for i, ip in enumerate(self.listRack):
                name = self.listRackNames[i] if i < len(self.listRackNames) else ip
                self.rackChoise.addItem(f"{name}  ({ip})")
            
            # Mettre à jour le combo des moteurs
            self.motorNameBox.clear()
            self.motorNameBox.addItem('Choose a motor')
            if self.listMotorName:
                self.motorNameBox.addItems(self.listMotorName)
        
        if self.zmqClient and self.zmqClient.isconnected:
            QMessageBox.information(self, "Connexion", "✅ Connexion réussie !")
        else:
            QMessageBox.warning(self, "Connexion", "❌ Échec de la connexion au serveur.")
    
    def showConnectionInfo(self):
        """Affiche les informations de connexion"""
        if self.zmqClient:
            status = "✅ Connecté" if self.zmqClient.isconnected else "❌ Déconnecté"
            server_status = "✅ Disponible" if self.zmqClient.server_available else "❌ Non disponible"
        else:
            status = "❌ Non initialisé"
            server_status = "❌ Non disponible"
        
        info = f"""
<b>Configuration Serveur ZMQ</b><br><br>
<b>Adresse:</b> {self.serverHost}<br>
<b>Port:</b> {self.serverPort}<br>
<b>URL complète:</b> tcp://{self.serverHost}:{self.serverPort}<br><br>
<b>État connexion:</b> {status}<br>
<b>État serveur:</b> {server_status}<br><br>
<b>Racks détectés:</b> {len(self.listRack)}<br>
<b>Moteurs disponibles:</b> {len(self.listMotorName)}
        """
        
        msgBox = QMessageBox(self)
        msgBox.setWindowTitle("Info Connexion")
        msgBox.setTextFormat(Qt.TextFormat.RichText)
        msgBox.setText(info)
        msgBox.setIcon(QMessageBox.Icon.Information)
        msgBox.exec()

    def ChangeIPRack(self):
        """Changement de rack sélectionné"""
        if not self.motRSAI or not self.listRack:
            return
            
        index = self.rackChoise.currentIndex()
        if 0 <= index < len(self.listRack):
            self.currentIP = self.listRack[index]
            print(f"🔧 Rack sélectionné: {self.currentIP}")
            
            # Mettre à jour la liste des moteurs
            self.motorNameBox.clear()
            self.motorNameBox.addItem('Choose a motor')
            
            self._updateMotorList()
            
            if self.listMotorName:
                self.motorNameBox.addItems(self.listMotorName)
                print(f"🔧 {len(self.listMotorName)} moteurs ajoutés au menu")

    def motorChange(self):
        """Changement de moteur sélectionné"""
        if self.motorNameBox.currentIndex() > 0:
            self.currentMotorNum = self.motorNameBox.currentIndex()
            
            if self.motA2V:
                self.currentMotorNum -= 1
                self.MOT = self.A2V.MOTORA2V(self.listMotor[self.currentMotorNum])
                self.stepmotor = self.MOT.step
                
            elif self.motRSAI and self.zmqClient:
                # Récupérer le step du moteur via ZMQ
                self.stepmotor = self.zmqClient.getStep(self.currentIP, self.currentMotorNum)
                if self.stepmotor == 0:
                    self.stepmotor = 1
                    
            self.unit()
            self._updatePositionLabel()
            
            print(f"Moteur sélectionné: {self.currentMotorNum}, step: {self.stepmotor}")

    def unit(self):
        """Changement d'unité"""
        self.indexUnit = self.unitBouton.currentIndex()
        
        if self.indexUnit == 0:  # step
            self.unitChange = 1
            self.unitName = 'step'
        elif self.indexUnit == 1:  # micron
            self.unitChange = float(1 * self.stepmotor)
            self.unitName = 'µm'
        elif self.indexUnit == 2:  # mm
            self.unitChange = float(1000 * self.stepmotor)
            self.unitName = 'mm'
        elif self.indexUnit == 3:  # ps (double passage: 1 micron = 6fs)
            self.unitChange = float(1 * self.stepmotor / 0.0066666666)
            self.unitName = 'ps'
        elif self.indexUnit == 4:  # degrés
            self.unitChange = 1 * self.stepmotor
            self.unitName = '°'
        
        if self.unitChange == 0:
            self.unitChange = 1
            
        self._updatePositionLabel()

    def Reset(self):
        self.shoot = 0
        self.table.setRowCount(0)
        self.Maxx = []
        self.Minn = []
        self.Summ = []
        self.Mean = []
        self.Xmax = []
        self.Ymax = []
        self.Xcmass = []
        self.Ycmass = []
        self.labelsVert = []
        self.posMotor = []
        self.SummThre = []
        self.USER1 = []

    def saveF(self):
        fname = QtGui.QFileDialog.getSaveFileName(self, "Save Measurements as txt file", self.path)
        self.path = os.path.dirname(str(fname[0]))
        f = open(str(fname[0]) + '.txt', 'w')
        f.write("\n".join(self.TableSauv))
        f.close()
        
    def openF(self):
        print('open not done yet')
    
    # Méthodes Zoom
    def ZoomMAX(self):
        self.open_widget(self.winZoomMax)
        self.winZoomMax.SetTITLE('MAX')
        self.winZoomMax.setZoom(self.maxx)
        
    def ZoomSUM(self):
        self.open_widget(self.winZoomSum)
        self.winZoomSum.SetTITLE('Sum')
        self.winZoomSum.setZoom(self.summ)
        
    def ZoomMEAN(self):
        self.open_widget(self.winZoomMean)
        self.winZoomMean.SetTITLE('Mean')
        self.winZoomMean.setZoom(self.moy)    
   
    def ZoomXmax(self):
        self.open_widget(self.winZoomXmax)
        self.winZoomXmax.SetTITLE('x max')
        self.winZoomXmax.setZoom(self.xmax)
        
    def ZoomYmax(self):
        self.open_widget(self.winZoomYmax)
        self.winZoomYmax.SetTITLE('y max')
        self.winZoomYmax.setZoom(self.ymax) 
    
    def ZoomCxmax(self):
        self.open_widget(self.winZoomCxmax)
        self.winZoomCxmax.SetTITLE('x center of mass')
        self.winZoomCxmax.setZoom(self.xcmass)   
    
    def ZoomCymax(self):
        self.open_widget(self.winZoomCymax)
        self.winZoomCymax.SetTITLE('y center of mass')
        self.winZoomCymax.setZoom(self.ycmass)
        
    def ZoomSUMThreshold(self):
        self.open_widget(self.winZoomSumThreshold)
        self.winZoomSumThreshold.SetTITLE('Sum threshold')
        self.winZoomSumThreshold.setZoom(self.summThre) 
    
    def ZoomUser1(self):
        self.open_widget(self.winZoomUser1)
        self.winZoomUser1.SetTITLE('User 1')
        self.winZoomUser1.setZoom(self.user1)

    # Méthodes Plot
    def PlotMAX(self):
        self.open_widget(self.winCoupeMax)
        self.winCoupeMax.SetTITLE('Plot Max')
        self.signalTrans['data'] = self.Maxx
        self.signalTrans['axis'] = self.posMotor
        self.signalTrans['label'] = self.label
        self.signalPlot.emit(self.signalTrans)
    
    def PlotMIN(self):
        self.open_widget(self.winCoupeMin)
        self.winCoupeMin.SetTITLE('Plot Min')
        self.signalTrans['data'] = self.Minn
        self.signalTrans['axis'] = self.posMotor
        self.signalTrans['label'] = self.label
        self.signalPlot.emit(self.signalTrans)
        
    def PlotXMAX(self):
        self.open_widget(self.winCoupeXmax)
        self.winCoupeXmax.SetTITLE('Plot X MAX')
        self.signalTrans['data'] = self.Xmax
        self.signalTrans['axis'] = self.posMotor
        self.signalTrans['label'] = self.label
        self.signalPlot.emit(self.signalTrans)
    
    def PlotYMAX(self):
        self.open_widget(self.winCoupeYmax)
        self.winCoupeYmax.SetTITLE('Plot Y MAX')
        self.signalTrans['data'] = self.Ymax
        self.signalTrans['axis'] = self.posMotor
        self.signalTrans['label'] = self.label
        self.signalPlot.emit(self.signalTrans)
     
    def PlotSUM(self):
        self.open_widget(self.winCoupeSum)
        self.winCoupeSum.SetTITLE('Plot Sum')
        self.signalTrans['data'] = self.Summ
        self.signalTrans['axis'] = self.posMotor
        self.signalTrans['label'] = self.label
        self.signalPlot.emit(self.signalTrans)
    
    def PlotMEAN(self):
        self.open_widget(self.winCoupeMean)
        self.winCoupeMean.SetTITLE('Plot Mean')
        self.signalTrans['data'] = self.Mean
        self.signalTrans['axis'] = self.posMotor
        self.signalTrans['label'] = self.label
        self.signalPlot.emit(self.signalTrans)
        
    def PlotXCMASS(self):
        self.open_widget(self.winCoupeXcmass)
        self.winCoupeXcmass.SetTITLE('Plot x center of mass')
        self.signalTrans['data'] = self.Xcmass
        self.signalTrans['axis'] = self.posMotor
        self.signalTrans['label'] = self.label
        self.signalPlot.emit(self.signalTrans)
    
    def PlotYCMASS(self):
        self.open_widget(self.winCoupeYcmass)
        self.winCoupeYcmass.SetTITLE('Plot Y center of mass')
        self.signalTrans['data'] = self.Ycmass
        self.signalTrans['axis'] = self.posMotor
        self.signalTrans['label'] = self.label
        self.signalPlot.emit(self.signalTrans)
    
    def PlotSUMTHRESHOLD(self):
        self.open_widget(self.winCoupeSumThreshold)
        self.winCoupeSumThreshold.SetTITLE('Plot Sum with Threshold')
        self.signalTrans['data'] = self.SummThre
        self.signalTrans['axis'] = self.posMotor
        self.signalTrans['label'] = self.label
        self.signalPlot.emit(self.signalTrans)
    
    def Threshold(self):
        threshold, ok = QInputDialog.getInt(self, 'Threshold Filter', 'Enter threshold value')
        if ok:
            self.ThresholdState = True
            self.threshold = threshold
            self.Reset()
            self.PlotMenu.addAction('Sum Threshold', self.PlotSUMTHRESHOLD)
            self.ZoomMenu.addAction('Sum with Threshold', self.ZoomSUMThreshold)
            self.Display(self.data)
        else:
            self.ThresholdState = False

    def PlotUSER1(self):
        self.open_widget(self.winCoupeUser1)
        self.winCoupeUser1.SetTITLE('User1')
        self.signalTrans['data'] = self.USER1
        self.signalTrans['axis'] = self.posMotor
        self.signalTrans['label'] = self.label
        self.signalPlot.emit(self.signalTrans)

    def Display(self, data):
        self.data = data[0]
        self.transx = data[1]
        self.transy = data[2]
        self.scalex = data[3]
        self.scaley = data[4]
        
        self.maxx = round(self.data.max(), 3)
        self.minn = round(self.data.min(), 3)
        self.summ = round(self.data.sum(), 3)
        self.moy = round(self.data.mean(), 3)
        self.user1 = round(self.FctUser1(), 3)
        self.date = time.strftime("%Y_%m_%d_%H_%M_%S")
        
        (self.xmax, self.ymax) = np.unravel_index(self.data.argmax(), self.data.shape)
        self.xmax = (self.xmax + self.transx) * self.scalex 
        self.ymax = (self.ymax + self.transy) * self.scaley 
        
        (self.xcmass, self.ycmass) = ndimage.center_of_mass(self.data)
        self.xcmass = (round(self.xcmass, 3) + self.transx) * self.scalex
        self.ycmass = (round(self.ycmass, 3) + self.transy) * self.scaley
        
        self.xs = self.data.shape[0]
        self.ys = self.data.shape[1]
        
        # Remplissage du tableau
        self.table.setRowCount(self.shoot + 1)
        self.table.setItem(self.shoot, 0, QTableWidgetItem(str(self.nomFichier)))
        self.table.setItem(self.shoot, 1, QTableWidgetItem(str(self.maxx)))
        self.table.setItem(self.shoot, 2, QTableWidgetItem(str(self.minn)))
        self.table.setItem(self.shoot, 3, QTableWidgetItem(str(self.xmax)))
        self.table.setItem(self.shoot, 4, QTableWidgetItem(str(self.ymax)))
        self.table.setItem(self.shoot, 5, QTableWidgetItem("{:.3e}".format(self.summ)))
        self.table.setItem(self.shoot, 6, QTableWidgetItem(str(self.moy)))
        self.table.setItem(self.shoot, 7, QTableWidgetItem((str(self.xs) + '*' + str(self.ys))))
        self.table.setItem(self.shoot, 8, QTableWidgetItem(str(self.xcmass)))
        self.table.setItem(self.shoot, 9, QTableWidgetItem(str(self.ycmass)))
        self.table.setItem(self.shoot, 10, QTableWidgetItem(str(self.user1)))

        # Gestion de la position moteur
        if self.motRSAI or self.motA2V:
            if self.motorNameBox.currentIndex() == 0:
                Posi = self.shoot
                self.label = 'Shoot'
            else:
                if self.motRSAI and self.zmqClient:
                    pos = self.zmqClient.getPosition(self.currentIP, self.currentMotorNum)
                    Posi = round(pos / self.unitChange, 2)
                    motorName = self.zmqClient.getMotorName(self.currentIP, self.currentMotorNum)
                    self.label = f"{motorName} ({self.unitName})"
                elif self.motA2V:
                    Posi = round(self.MOT.position() / self.unitChange, 2)
                    self.label = f"{self.MOT.name} ({self.unitName})"
                else:
                    Posi = self.shoot
                    self.label = 'Shoot'
        else:
            Posi = self.shoot
            self.label = 'Shoot'
            
        self.posMotor.append(Posi)    
        self.table.resizeColumnsToContents()
        self.labelsVert.append('%s' % self.shoot)
        
        # Gestion du threshold
        if self.ThresholdState:
            dataCor = np.where(self.data < self.threshold, 0, self.data)
            self.summThre = round(dataCor.sum(), 3)
            self.SummThre.append(self.summThre)
            
            if self.motRSAI or self.motA2V:
                self.table.setColumnCount(14)
                self.table.setHorizontalHeaderLabels(('File', 'Max', 'Min', 'x max', 'y max', 'Sum', 'Mean', 'Size', 'x c.mass', 'y c.mass', 'Sum Thr', 'user1', 'Motor', 'date'))
                self.table.setItem(self.shoot, 12, QTableWidgetItem(str(Posi)))
                self.table.setItem(self.shoot, 13, QTableWidgetItem(str(self.date)))
            else:
                self.table.setColumnCount(13)
                self.table.setHorizontalHeaderLabels(('File', 'Max', 'Min', 'x max', 'y max', 'Sum', 'Mean', 'Size', 'x c.mass', 'y c.mass', 'Sum Thr', 'user1', 'date'))
                self.table.setItem(self.shoot, 12, QTableWidgetItem(str(self.date)))
            self.table.setItem(self.shoot, 10, QTableWidgetItem("{:.3e}".format(self.summThre)))
            
        else:
            if self.motRSAI or self.motA2V:
                self.table.setHorizontalHeaderLabels(('File', 'Max', 'Min', 'x max', 'y max', 'Sum', 'Mean', 'Size', 'x c.mass', 'y c.mass', 'user1', 'Motor', 'date'))
                self.table.setColumnCount(13)
                self.table.setItem(self.shoot, 11, QTableWidgetItem(str(Posi)))
                self.table.setItem(self.shoot, 12, QTableWidgetItem(str(self.date)))
            else:
                self.table.setHorizontalHeaderLabels(('File', 'Max', 'Min', 'x max', 'y max', 'Sum', 'Mean', 'Size', 'x c.mass', 'y c.mass', 'user1', 'date'))
                self.table.setColumnCount(12)
                self.table.setItem(self.shoot, 11, QTableWidgetItem(str(self.date)))
        
        self.table.selectRow(self.shoot)
        
        # Mise à jour des listes
        self.Maxx.append(self.maxx)
        self.Minn.append(self.minn)
        self.Summ.append(self.summ)
        self.Mean.append(self.moy)
        self.Xmax.append(self.xmax)
        self.Ymax.append(self.ymax)
        self.Xcmass.append(self.xcmass)
        self.Ycmass.append(self.ycmass)
        self.USER1.append(self.user1)

        self.table.setVerticalHeaderLabels(self.labelsVert)

        # Mise à jour des fenêtres de plot ouvertes
        if self.winCoupeMax.isWinOpen:
            self.PlotMAX()
        if self.winCoupeMin.isWinOpen:
            self.PlotMIN()
        if self.winCoupeXmax.isWinOpen:
            self.PlotXMAX()
        if self.winCoupeYmax.isWinOpen:
            self.PlotYMAX()
        if self.winCoupeSum.isWinOpen:
            self.PlotSUM()
        if self.winCoupeMean.isWinOpen:
            self.PlotMEAN()
        if self.winCoupeXcmass.isWinOpen:
            self.PlotXCMASS()
        if self.winCoupeYcmass.isWinOpen:
            self.PlotYCMASS()
        if self.winCoupeSumThreshold.isWinOpen:
            self.PlotSUMTHRESHOLD()
        if self.winCoupeUser1.isWinOpen:
            self.PlotUSER1()

        # Mise à jour des fenêtres zoom ouvertes
        if self.winZoomMax.isWinOpen:
            self.ZoomMAX()
        if self.winZoomSum.isWinOpen:
            self.ZoomSUM()
        if self.winZoomMean.isWinOpen:
            self.ZoomMEAN() 
        if self.winZoomXmax.isWinOpen:
            self.ZoomXmax()
        if self.winZoomYmax.isWinOpen:
            self.ZoomYmax()
        if self.winZoomCxmax.isWinOpen:
            self.ZoomCxmax()
        if self.winZoomCymax.isWinOpen:
            self.ZoomCymax()   
        if self.winZoomSumThreshold.isWinOpen:
            self.ZoomSUMThreshold()
        if self.winZoomUser1.isWinOpen:
            self.ZoomUser1()
            
        # Mise à jour position
        self._updatePositionLabel()
        
        self.shoot += 1
      
    def closeEvent(self, event):
        """Fermeture de la fenêtre"""
        self.isWinOpen = False
        self.shoot = 0
        self.TableSauv = ['file,Max,Min,x Max,y max,Sum,Mean,Size,x c.mass,y c.mass']
        
        # Arrêter le timer de position
        if hasattr(self, 'positionTimer'):
            self.positionTimer.stop()
        
        # Fermeture des fenêtres de graphiques
        for win in [self.winCoupeMax, self.winCoupeMin, self.winCoupeXmax, 
                    self.winCoupeYmax, self.winCoupeSum, self.winCoupeMean,
                    self.winCoupeXcmass, self.winCoupeYcmass, self.winCoupeSumThreshold]:
            if win.isWinOpen:
                win.close()
        
        # Fermeture de la connexion ZMQ
        if self.zmqClient:
            self.zmqClient.close()
            
        time.sleep(0.1)
        event.accept()

    def open_widget(self, fene):
        """Ouverture widget supplémentaire"""
        if fene.isWinOpen is False:
            fene.setup
            fene.isWinOpen = True
            fene.show()
        else:
            fene.showNormal()

    def FctUser1(self):
        """Fonction utilisateur personnalisable"""
        a = 0
        return a


if __name__ == "__main__":
    appli = QApplication(sys.argv)
    appli.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
    e = MEAS(motRSAI=True)
    e.show()
    appli.exec()